@@ -3,6 +3,7 @@
 #define ____CLASS_STANDARD_MYSQL_THREAD_H
 #include "proxysql.h"
 #include "cpp.h"
+#include "MySQL_Variables.h"
 #ifdef IDLE_THREADS
 #include <sys/epoll.h>
 #endif // IDLE_THREADS
@@ -17,28 +18,16 @@
 #define SQLITE_HOSTGROUP -4
 
 
-#define MYSQL_DEFAULT_SQL_MODE	""
-#define MYSQL_DEFAULT_TIME_ZONE	"SYSTEM"
-#define MYSQL_DEFAULT_ISOLATION_LEVEL	"READ COMMITTED"
-#define MYSQL_DEFAULT_TRANSACTION_READ	"WRITE"
 #define MYSQL_DEFAULT_TX_ISOLATION	"READ-COMMITTED"
-#define MYSQL_DEFAULT_CHARACTER_SET_RESULTS	"NULL"
-#define MYSQL_DEFAULT_SESSION_TRACK_GTIDS	"OFF"
-#define MYSQL_DEFAULT_SQL_AUTO_IS_NULL	"OFF"
-#define MYSQL_DEFAULT_SQL_SELECT_LIMIT	"DEFAULT"
-#define MYSQL_DEFAULT_SQL_SAFE_UPDATES	"OFF"
+#define MYSQL_DEFAULT_SESSION_TRACK_GTIDS      "OFF"
 #define MYSQL_DEFAULT_COLLATION_CONNECTION	""
 #define MYSQL_DEFAULT_NET_WRITE_TIMEOUT	"60"
 #define MYSQL_DEFAULT_MAX_JOIN_SIZE	"18446744073709551615"
 
-static unsigned int near_pow_2 (unsigned int n) {
-  unsigned int i = 1;
-  while (i < n) i <<= 1;
-  return i ? i : n;
-}
+extern class MySQL_Variables mysql_variables;
 
 #ifdef IDLE_THREADS
-typedef struct __attribute__((aligned(CACHE_LINE_SIZE))) _conn_exchange_t {
+typedef struct __attribute__((aligned(64))) _conn_exchange_t {
 	pthread_mutex_t mutex_idles;
 	PtrArray *idle_mysql_sessions;
 	pthread_mutex_t mutex_resumes;
@@ -58,119 +47,28 @@ typedef struct _kill_queue_t {
 } kill_queue_t;
 
 class ProxySQL_Poll {
+	private:
+	void shrink();
+	void expand(unsigned int more);
 
-  private:
-  void shrink() {
-    unsigned int new_size=near_pow_2(len+1);
-    fds=(struct pollfd *)realloc(fds,new_size*sizeof(struct pollfd));
-    myds=(MySQL_Data_Stream **)realloc(myds,new_size*sizeof(MySQL_Data_Stream *));
-		last_recv=(unsigned long long *)realloc(last_recv,new_size*sizeof(unsigned long long));
-		last_sent=(unsigned long long *)realloc(last_sent,new_size*sizeof(unsigned long long));
-    size=new_size;
-  };
-  void expand(unsigned int more) {
-    if ( (len+more) > size ) {
-      unsigned int new_size=near_pow_2(len+more);
-      fds=(struct pollfd *)realloc(fds,new_size*sizeof(struct pollfd));
-      myds=(MySQL_Data_Stream **)realloc(myds,new_size*sizeof(MySQL_Data_Stream *));
-			last_recv=(unsigned long long *)realloc(last_recv,new_size*sizeof(unsigned long long));
-			last_sent=(unsigned long long *)realloc(last_sent,new_size*sizeof(unsigned long long));
-      size=new_size;
-    }
-  };
-
-  public:
+	public:
 	unsigned int poll_timeout;
 	unsigned long loops;
 	StatCounters *loop_counters;
-  unsigned int len;
-  unsigned int size;
-  struct pollfd *fds;
-  MySQL_Data_Stream **myds;
+	unsigned int len;
+	unsigned int size;
+	struct pollfd *fds;
+	MySQL_Data_Stream **myds;
 	unsigned long long *last_recv;
 	unsigned long long *last_sent;
 	volatile int pending_listener_add;
 	volatile int pending_listener_del;
 
-  ProxySQL_Poll() {
-#ifdef PROXYSQL_STATSCOUNTERS_NOLOCK
-		loop_counters=new StatCounters(15,10);
-#else
-		loop_counters=new StatCounters(15,10,false);
-#endif
-		poll_timeout=0;
-		loops=0;
-		len=0;
-		pending_listener_add=0;
-		pending_listener_del=0;
-    size=MIN_POLL_LEN;
-    fds=(struct pollfd *)malloc(size*sizeof(struct pollfd));
-    myds=(MySQL_Data_Stream **)malloc(size*sizeof(MySQL_Data_Stream *));
-		last_recv=(unsigned long long *)malloc(size*sizeof(unsigned long long));
-		last_sent=(unsigned long long *)malloc(size*sizeof(unsigned long long));
-  };
-
-  ~ProxySQL_Poll() {
-    unsigned int i;
-    for (i=0;i<len;i++) {
-      if (
-				myds[i] && // fix bug #278 . This should be caused by not initialized datastreams used to ping the backend
-				myds[i]->myds_type==MYDS_LISTENER) {
-        delete myds[i];
-      }
-    }
-    free(myds);
-    free(fds);
-		free(last_recv);
-		free(last_sent);
-		delete loop_counters;
-  };
-
-  void add(uint32_t _events, int _fd, MySQL_Data_Stream *_myds, unsigned long long sent_time) {
-    if (len==size) {
-      expand(1);
-    }
-    myds[len]=_myds;
-    fds[len].fd=_fd;
-    fds[len].events=_events;
-    fds[len].revents=0;
-		if (_myds) {
-			_myds->mypolls=this;
-			_myds->poll_fds_idx=len;  // fix a serious bug
-		}
-    last_recv[len]=monotonic_time();
-    last_sent[len]=sent_time;
-    len++;
-  };
-
-  void remove_index_fast(unsigned int i) {
-		if ((int)i==-1) return;
-		myds[i]->poll_fds_idx=-1; // this prevents further delete
-    if (i != (len-1)) {
-      myds[i]=myds[len-1];
-      fds[i].fd=fds[len-1].fd;
-      fds[i].events=fds[len-1].events;
-      fds[i].revents=fds[len-1].revents;
-			myds[i]->poll_fds_idx=i;  // fix a serious bug
-    	last_recv[i]=last_recv[len-1];
-    	last_sent[i]=last_sent[len-1];
-    }
-    len--;
-    if ( ( len>MIN_POLL_LEN ) && ( size > len*MIN_POLL_DELETE_RATIO ) ) {
-      shrink();
-    }
-  };  
-
-	int find_index(int fd) {
-		unsigned int i;
-		for (i=0; i<len; i++) {
-			if (fds[i].fd==fd) {
-				return i;
-			}
-		}
-		return -1;
-	}
-
+	ProxySQL_Poll();
+	~ProxySQL_Poll();
+	void add(uint32_t _events, int _fd, MySQL_Data_Stream *_myds, unsigned long long sent_time);
+	void remove_index_fast(unsigned int i);
+	int find_index(int fd);
 };
 
 
@@ -268,11 +166,14 @@ class MySQL_Thread
 		unsigned long long hostgroup_locked_set_cmds;
 		unsigned long long hostgroup_locked_queries;
 		unsigned long long aws_aurora_replicas_skipped_during_query;
+		unsigned long long automatic_detected_sqli;
+		unsigned long long whitelisted_sqli_fingerprint;
 		unsigned int active_transactions;
 	} status_variables;
 
 	struct {
 		int min_num_servers_lantency_awareness;
+		int aurora_max_lag_ms_only_read_from_replicas;
 		bool stats_time_backend_query;
 		bool stats_time_query_processor;
 		bool query_cache_stores_empty_result;
@@ -368,6 +269,7 @@ class MySQL_Threads_Handler
 		int monitor_groupreplication_healthcheck_interval;
 		int monitor_groupreplication_healthcheck_timeout;
 		int monitor_groupreplication_healthcheck_max_timeout_count;
+		int monitor_groupreplication_max_transactions_behind_count;
 		int monitor_galera_healthcheck_interval;
 		int monitor_galera_healthcheck_timeout;
 		int monitor_galera_healthcheck_max_timeout_count;
@@ -386,6 +288,7 @@ class MySQL_Threads_Handler
 		int shun_recovery_time_sec;
 		int query_retries_on_failure;
 		bool client_multi_statements;
+		bool connection_warming;
 		int connect_retries_on_failure;
 		int connect_retries_delay;
 		int connection_delay_multiplex_ms;
@@ -403,7 +306,8 @@ class MySQL_Threads_Handler
 		char *interfaces;
 		char *server_version;
 		char *keep_multiplexing_variables;
-		uint8_t default_charset;
+		unsigned int default_charset;
+		unsigned int handle_unknown_charset;
 		bool servers_stats;
 		bool commands_stats;
 		bool query_digests;
@@ -418,12 +322,15 @@ class MySQL_Threads_Handler
 		bool client_found_rows;
 		bool multiplexing;
 //		bool stmt_multiplexing;
+		bool log_unhealthy_connections;
 		bool forward_autocommit;
 		bool enforce_autocommit_on_reads;
 		bool autocommit_false_not_reusable;
 		bool autocommit_false_is_transaction;
 		bool verbose_query_error;
 		int max_allowed_packet;
+		bool automatic_detect_sqli;
+		bool firewall_whitelist_enabled;
 		bool use_tcp_keepalive;
 		int tcp_keepalive_time;
 		int throttle_connections_per_sec_to_hostgroup;
@@ -454,19 +361,10 @@ class MySQL_Threads_Handler
 		char *init_connect;
 		char *ldap_user_variable;
 		char *add_ldap_user_comment;
-		char *default_sql_mode;
-		char *default_time_zone;
-		char *default_isolation_level;
-		char *default_transaction_read;
 		char *default_tx_isolation;
-		char *default_character_set_results;
 		char *default_session_track_gtids;
-		char *default_sql_auto_is_null;
-		char *default_sql_select_limit;
-		char *default_sql_safe_updates;
-		char *default_collation_connection;
-		char *default_net_write_timeout;
-		char *default_max_join_size;
+		char *default_variables[SQL_NAME_LAST];
+		char *firewall_whitelist_errormsg;
 #ifdef DEBUG
 		bool session_debug;
 #endif /* DEBUG */
@@ -487,6 +385,7 @@ class MySQL_Threads_Handler
 		char * ssl_p2s_cipher;
 		int query_cache_size_MB;
 		int min_num_servers_lantency_awareness;
+		int aurora_max_lag_ms_only_read_from_replicas;
 		bool stats_time_backend_query;
 		bool stats_time_query_processor;
 		bool query_cache_stores_empty_result;
@@ -506,7 +405,7 @@ class MySQL_Threads_Handler
  	void wrunlock();
 	void commit();
 	char *get_variable(char *name);
-	bool set_variable(char *name, char *value);
+	bool set_variable(char *name, const char *value);
 	char **get_variables_list();
 	bool has_variable(const char * name);
 
@@ -514,7 +413,7 @@ class MySQL_Threads_Handler
 	~MySQL_Threads_Handler();
 	
 	char *get_variable_string(char *name);
-	uint8_t get_variable_uint8(char *name);
+	unsigned int get_variable_uint(char *name);
 	uint16_t get_variable_uint16(char *name);
 	int get_variable_int(const char *name);
 	void print_version();
@@ -567,6 +466,8 @@ class MySQL_Threads_Handler
 	unsigned long long get_hostgroup_locked_set_cmds();
 	unsigned long long get_hostgroup_locked_queries();
 	unsigned long long get_aws_aurora_replicas_skipped_during_query();
+	unsigned long long get_automatic_detected_sqli();
+	unsigned long long get_whitelisted_sqli_fingerprint();
 	unsigned long long get_backend_lagging_during_query();
 	unsigned long long get_backend_offline_during_query();
 	unsigned long long get_queries_with_max_lag_ms();
