@@ -16,7 +16,6 @@ class SQLite3_Session {
 class SQLite3_Server {
 	private:
 	volatile int main_shutdown;
-	SQLite3DB *sessdb;
 	int main_poll_nfds;
 	struct pollfd *main_poll_fds;
 	int *main_callback_func;
@@ -52,6 +51,7 @@ class SQLite3_Server {
 	void check_and_build_standard_tables(SQLite3DB *db, std::vector<table_def_t *> *tables_defs);
 #endif // TEST_AURORA || TEST_GALERA
 	public:
+	SQLite3DB *sessdb;
 #ifdef TEST_AURORA
 	unsigned int cur_aurora_writer[3];
 	unsigned int num_aurora_servers[3];
@@ -70,7 +70,7 @@ class SQLite3_Server {
 #endif // TEST_GALERA
 #ifdef TEST_GROUPREP
 	pthread_mutex_t grouprep_mutex;
-	void populate_grouprep_table(MySQL_Session *sess);
+	void populate_grouprep_table(MySQL_Session *sess, int txs_behind = 0);
 	void init_grouprep_ifaces_string(std::string& s);
 #endif // TEST_GROUPREP
 	SQLite3_Server();
@@ -83,7 +83,7 @@ class SQLite3_Server {
 	bool init();
 	void wrlock();
 	void wrunlock();
-	void send_MySQL_OK(MySQL_Protocol *myprot, char *msg, int rows=0);
+	void send_MySQL_OK(MySQL_Protocol *myprot, char *msg, int rows=0, uint16_t status=2);
 	void send_MySQL_ERR(MySQL_Protocol *myprot, char *msg);
 };
 #endif // CLASS_PROXYSQL_SQLITE3_SERVER_H
