@@ -4,6 +4,9 @@
 #include "proxysql.h"
 #include "cpp.h"
 
+#include "../deps/json/json.hpp"
+using json = nlohmann::json;
+
 #define STATUS_MYSQL_CONNECTION_TRANSACTION          0x00000001
 #define STATUS_MYSQL_CONNECTION_COMPRESSION          0x00000002
 #define STATUS_MYSQL_CONNECTION_USER_VARIABLE        0x00000004
@@ -16,6 +19,22 @@
 #define STATUS_MYSQL_CONNECTION_FOUND_ROWS           0x00000200
 #define STATUS_MYSQL_CONNECTION_NO_BACKSLASH_ESCAPES 0x00000400
 
+class Variable {
+public:
+	char *value = (char*)"";
+	void fill_server_internal_session(json &j, int conn_num, int idx);
+	void fill_client_internal_session(json &j, int idx);
+	static const char set_name[SQL_NAME_LAST][64];
+	static const char proxysql_internal_session_name[SQL_NAME_LAST][64];
+};
+
+enum charset_action {
+	UNKNOWN,
+	NAMES,
+	CHARSET,
+	CONNECT_START
+};
+
 class MySQL_Connection_userinfo {
 	private:
 	uint64_t compute_hash();
@@ -40,60 +59,28 @@ class MySQL_Connection {
 	public:
 	struct {
 		char *server_version;
-		char *sql_mode;
-		char *time_zone;
-		uint32_t sql_mode_int;
-		uint32_t time_zone_int;
-		uint32_t character_set_results_int;
-		uint32_t isolation_level_int;
-		uint32_t transaction_read_int;
-		uint32_t tx_isolation_int;
 		uint32_t session_track_gtids_int;
-		uint32_t sql_auto_is_null_int;
-		uint32_t sql_select_limit_int;
-		uint32_t sql_safe_updates_int;
-		uint32_t collation_connection_int;
-		uint32_t net_write_timeout_int;
-		uint32_t max_join_size_int;
 		uint32_t max_allowed_pkt;
 		uint32_t server_capabilities;
 		uint32_t client_flag;
 		unsigned int compression_min_length;
 		char *init_connect;
 		bool init_connect_sent;
-		char * character_set_results;
-		char * isolation_level;
-		char * transaction_read;
-		char * tx_isolation;
 		char * session_track_gtids;
-		char * sql_auto_is_null;
-		char * sql_select_limit;
-		char * sql_safe_updates;
-		char * collation_connection;
-		char * net_write_timeout;
-		char * max_join_size;
-		bool isolation_level_sent;
-		bool tx_isolation_sent;
-		bool transaction_read_sent;
-		bool character_set_results_sent;
-		bool session_track_gtids_sent;
-		bool sql_auto_is_null_sent;
-		bool sql_select_limit_sent;
-		bool sql_safe_updates_sent;
-		bool collation_connection_sent;
-		bool net_write_timeout_sent;
-		bool max_join_size_sent;
-		bool sql_mode_sent;
 		char *ldap_user_variable;
 		char *ldap_user_variable_value;
+		bool session_track_gtids_sent;
 		bool ldap_user_variable_sent;
 		uint8_t protocol_version;
-		uint8_t charset;
-		uint8_t sql_log_bin;
 		int8_t last_set_autocommit;
 		bool autocommit;
 		bool no_backslash_escapes;
 	} options;
+
+	Variable variables[SQL_NAME_LAST];
+	uint32_t var_hash[SQL_NAME_LAST];
+	bool var_absent[SQL_NAME_LAST] = {false};
+
 	struct {
 		unsigned long length;
 		char *ptr;
@@ -149,7 +136,7 @@ class MySQL_Connection {
 	~MySQL_Connection();
 	bool set_autocommit(bool);
 	bool set_no_backslash_escapes(bool);
-	uint8_t set_charset(uint8_t);
+	unsigned int set_charset(unsigned int, enum charset_action);
 
 	void set_status_transaction(bool);
 	void set_status_compression(bool);
@@ -189,6 +176,8 @@ class MySQL_Connection {
 	void store_result_cont(short event);
 	void initdb_start();
 	void initdb_cont(short event);
+	void set_option_start();
+	void set_option_cont(short event);
 	void set_query(char *stmt, unsigned long length);
 	MDB_ASYNC_ST handler(short event);
 	void next_event(MDB_ASYNC_ST new_st);
@@ -197,10 +186,11 @@ class MySQL_Connection {
 	int async_change_user(short event);
 	int async_select_db(short event);
 	int async_set_autocommit(short event, bool);
-	int async_set_names(short event, uint8_t nr);
+	int async_set_names(short event, unsigned int nr);
 	int async_send_simple_command(short event, char *stmt, unsigned long length); // no result set expected
 	int async_query(short event, char *stmt, unsigned long length, MYSQL_STMT **_stmt=NULL, stmt_execute_metadata_t *_stmt_meta=NULL);
 	int async_ping(short event);
+	int async_set_option(short event, bool mask);
 
 	void stmt_prepare_start();
 	void stmt_prepare_cont(short event);
@@ -237,5 +227,6 @@ class MySQL_Connection {
 	void reduce_auto_increment_delay_token() { if (auto_increment_delay_token) auto_increment_delay_token--; };
 
 	bool match_tracked_options(MySQL_Connection *c);
+	unsigned long get_mysql_thread_id() { return mysql ? mysql->thread_id : 0; }
 };
 #endif /* __CLASS_MYSQL_CONNECTION_H */
