@@ -10,6 +10,14 @@
 #ifndef PROXYSQL_ENUMS
 #define PROXYSQL_ENUMS
 
+enum MySerStatus {
+	MYSQL_SERVER_STATUS_ONLINE,
+	MYSQL_SERVER_STATUS_SHUNNED,
+	MYSQL_SERVER_STATUS_OFFLINE_SOFT,
+	MYSQL_SERVER_STATUS_OFFLINE_HARD,
+	MYSQL_SERVER_STATUS_SHUNNED_REPLICATION_LAG
+};
+
 enum log_event_type {
 	PROXYSQL_COM_QUERY,
 	PROXYSQL_MYSQL_AUTH_OK,
@@ -77,6 +85,11 @@ enum MDB_ASYNC_ST { // MariaDB Async State Machine
 	ASYNC_INITDB_END,
 	ASYNC_INITDB_SUCCESSFUL,
 	ASYNC_INITDB_FAILED,
+	ASYNC_SET_OPTION_START,
+	ASYNC_SET_OPTION_CONT,
+	ASYNC_SET_OPTION_END,
+	ASYNC_SET_OPTION_FAILED,
+	ASYNC_SET_OPTION_SUCCESSFUL,
 	ASYNC_STMT_PREPARE_START,
 	ASYNC_STMT_PREPARE_CONT,
 	ASYNC_STMT_PREPARE_END,
@@ -133,6 +146,30 @@ enum MySQL_DS_type {
 };
 
 
+enum variable_name {
+	SQL_CHARACTER_SET,
+	SQL_CHARACTER_ACTION,
+	SQL_SET_NAMES,
+	SQL_SAFE_UPDATES,
+	SQL_SELECT_LIMIT,
+	SQL_SQL_MODE,
+	SQL_TIME_ZONE,
+	SQL_CHARACTER_SET_RESULTS,
+	SQL_CHARACTER_SET_CONNECTION,
+	SQL_CHARACTER_SET_CLIENT,
+	SQL_CHARACTER_SET_DATABASE,
+	SQL_ISOLATION_LEVEL,
+	SQL_TRANSACTION_READ,
+	SQL_SQL_AUTO_IS_NULL,
+	SQL_COLLATION_CONNECTION,
+	SQL_NET_WRITE_TIMEOUT,
+	SQL_MAX_JOIN_SIZE,
+	SQL_LOG_BIN,
+	SQL_WSREP_SYNC_WAIT,
+	SQL_GROUP_CONCAT_MAX_LEN,
+	SQL_NAME_LAST
+};
+
 enum session_status {
 	CONNECTING_CLIENT,
 	CONNECTING_SERVER,
@@ -143,32 +180,41 @@ enum session_status {
 	PROCESSING_QUERY,
 	CHANGING_SCHEMA,
 	CHANGING_CHARSET,
+	SETTING_CHARSET,
 	CHANGING_AUTOCOMMIT,
 	CHANGING_USER_CLIENT,
 	CHANGING_USER_SERVER,
 	RESETTING_CONNECTION,
 	SETTING_INIT_CONNECT,
 	SETTING_LDAP_USER_VARIABLE,
-	SETTING_SQL_LOG_BIN,
-	SETTING_SQL_MODE,
-	SETTING_TIME_ZONE,
 	SETTING_ISOLATION_LEVEL,
 	SETTING_TRANSACTION_READ,
-	SETTING_TX_ISOLATION,
-	SETTING_CHARACTER_SET_RESULTS,
 	SETTING_SESSION_TRACK_GTIDS,
-	SETTING_SQL_AUTO_IS_NULL,
-	SETTING_SQL_SELECT_LIMIT,
-	SETTING_SQL_SAFE_UPDATES,
-	SETTING_COLLATION_CONNECTION,
-	SETTING_NET_WRITE_TIMEOUT,
-	SETTING_MAX_JOIN_SIZE,
+	SETTING_MULTI_STMT,
 	FAST_FORWARD,
 	PROCESSING_STMT_PREPARE,
 	PROCESSING_STMT_EXECUTE,
+	SETTING_VARIABLE,
+	SETTING_MULTIPLE_VARIABLES,
+	SETTING_SET_NAMES,
 	NONE
 };
 
+#ifdef __cplusplus
+typedef struct {
+	enum variable_name idx;     // index number
+	enum session_status status; // what status should be changed after setting this variables
+	bool quote;                 // if the variable needs to be quoted
+	bool set_transaction;       // if related to SET TRANSACTION statement . if false , it will be execute "SET varname = varvalue" . If true, "SET varname varvalue"
+	bool special_handling;      // if true, some special handling is required
+	bool is_number;				// if true, the variable is a number. Special cases should be checked
+	bool is_bool;				// if true, the variable is a boolean. Special cases should be checked
+	char * set_variable_name;   // what variable name (or string) will be used when setting it to backend
+	char * internal_variable_name; // variable name as displayed in admin , WITHOUT "default_"
+	char * default_value;       // default value
+} mysql_variable_st;
+#endif
+
 enum mysql_data_stream_status {
 	STATE_NOT_INITIALIZED,
 	STATE_NOT_CONNECTED,
@@ -327,6 +373,13 @@ enum MYSQL_COM_QUERY_command {
 	MYSQL_COM_QUERY___NONE // Special marker.
 };
 
+enum handle_unknown_charset {
+	HANDLE_UNKNOWN_CHARSET__DISCONNECT_CLIENT,
+	HANDLE_UNKNOWN_CHARSET__REPLACE_WITH_DEFAULT_VERBOSE,
+	HANDLE_UNKNOWN_CHARSET__REPLACE_WITH_DEFAULT,
+	HANDLE_UNKNOWN_CHARSET__MAX_HANDLE_VALUE
+};
+
 #endif /* PROXYSQL_ENUMS */
 
 
@@ -379,6 +432,15 @@ class SQLite3_result;
 class stmt_execute_metadata_t;
 class MySQL_STMTs_meta;
 class MySQL_HostGroups_Manager;
+class ProxySQL_HTTP_Server;
+class MySQL_STMTs_local_v14;
+class MySQL_STMT_Global_info;
+class StmtLongDataHandler;
+class ProxySQL_Cluster;
+class MySQL_ResultSet;
+class Query_Processor_Output;
+class MySrvC;
+class Web_Interface_plugin;
 #endif /* PROXYSQL_CLASSES */
 //#endif /* __cplusplus */
 
@@ -498,6 +560,7 @@ struct _global_variables_t {
 
 	bool has_debug;
 	bool idle_threads;
+	bool version_check;
 
 	volatile int shutdown;
 	bool nostart;
@@ -514,7 +577,6 @@ struct _global_variables_t {
 
 	int merge_configfile_db;
 
-	int core_dump_file_size;
 	int stack_size;
 	char *proxy_admin_socket;
 	char *proxy_mysql_bind;
@@ -621,20 +683,12 @@ __thread char *mysql_thread___server_version;
 __thread char *mysql_thread___keep_multiplexing_variables;
 __thread char *mysql_thread___init_connect;
 __thread char *mysql_thread___ldap_user_variable;
-__thread char *mysql_thread___default_sql_mode;
-__thread char *mysql_thread___default_time_zone;
-__thread char *mysql_thread___default_isolation_level;
-__thread char *mysql_thread___default_transaction_read;
 __thread char *mysql_thread___default_tx_isolation;
-__thread char *mysql_thread___default_character_set_results;
 __thread char *mysql_thread___default_session_track_gtids;
-__thread char *mysql_thread___default_sql_auto_is_null;
-__thread char *mysql_thread___default_sql_select_limit;
-__thread char *mysql_thread___default_sql_safe_updates;
-__thread char *mysql_thread___default_collation_connection;
-__thread char *mysql_thread___default_net_write_timeout;
-__thread char *mysql_thread___default_max_join_size;
+__thread char *mysql_thread___firewall_whitelist_errormsg;
 __thread int mysql_thread___max_allowed_packet;
+__thread bool mysql_thread___automatic_detect_sqli;
+__thread bool mysql_thread___firewall_whitelist_enabled;
 __thread bool mysql_thread___use_tcp_keepalive;
 __thread int mysql_thread___tcp_keepalive_time;
 __thread int mysql_thread___throttle_connections_per_sec_to_hostgroup;
@@ -672,13 +726,15 @@ __thread int mysql_thread___set_query_lock_on_hostgroup;
 __thread int mysql_thread___reset_connection_algorithm;
 __thread uint32_t mysql_thread___server_capabilities;
 __thread int mysql_thread___auto_increment_delay_multiplex;
-__thread uint8_t mysql_thread___default_charset;
+__thread unsigned int mysql_thread___handle_unknown_charset;
 __thread int mysql_thread___poll_timeout;
 __thread int mysql_thread___poll_timeout_on_failure;
+__thread bool mysql_thread___connection_warming;
 __thread bool mysql_thread___have_compress;
 __thread bool mysql_thread___have_ssl;
 __thread bool mysql_thread___client_found_rows;
 __thread bool mysql_thread___multiplexing;
+__thread bool mysql_thread___log_unhealthy_connections;
 __thread bool mysql_thread___forward_autocommit;
 __thread bool mysql_thread___enforce_autocommit_on_reads;
 __thread bool mysql_thread___autocommit_false_not_reusable;
@@ -702,6 +758,7 @@ __thread bool mysql_thread___session_idle_show_processlist;
 __thread bool mysql_thread___sessions_sort;
 __thread bool mysql_thread___kill_backend_connection_when_disconnect;
 __thread bool mysql_thread___client_session_track_gtid;
+__thread char * mysql_thread___default_variables[SQL_NAME_LAST];
 
 /* variables used for Query Cache */
 __thread int mysql_thread___query_cache_size_MB;
@@ -740,6 +797,7 @@ __thread int mysql_thread___monitor_replication_lag_timeout;
 __thread int mysql_thread___monitor_groupreplication_healthcheck_interval;
 __thread int mysql_thread___monitor_groupreplication_healthcheck_timeout;
 __thread int mysql_thread___monitor_groupreplication_healthcheck_max_timeout_count;
+__thread int mysql_thread___monitor_groupreplication_max_transactions_behind_count;
 __thread int mysql_thread___monitor_galera_healthcheck_interval;
 __thread int mysql_thread___monitor_galera_healthcheck_timeout;
 __thread int mysql_thread___monitor_galera_healthcheck_max_timeout_count;
@@ -770,20 +828,12 @@ extern __thread char *mysql_thread___server_version;
 extern __thread char *mysql_thread___keep_multiplexing_variables;
 extern __thread char *mysql_thread___init_connect;
 extern __thread char *mysql_thread___ldap_user_variable;
-extern __thread char *mysql_thread___default_sql_mode;
-extern __thread char *mysql_thread___default_time_zone;
-extern __thread char *mysql_thread___default_isolation_level;
-extern __thread char *mysql_thread___default_transaction_read;
 extern __thread char *mysql_thread___default_tx_isolation;
-extern __thread char *mysql_thread___default_character_set_results;
 extern __thread char *mysql_thread___default_session_track_gtids;
-extern __thread char *mysql_thread___default_sql_auto_is_null;
-extern __thread char *mysql_thread___default_sql_select_limit;
-extern __thread char *mysql_thread___default_sql_safe_updates;
-extern __thread char *mysql_thread___default_collation_connection;
-extern __thread char *mysql_thread___default_net_write_timeout;
-extern __thread char *mysql_thread___default_max_join_size;
+extern __thread char *mysql_thread___firewall_whitelist_errormsg;
 extern __thread int mysql_thread___max_allowed_packet;
+extern __thread bool mysql_thread___automatic_detect_sqli;
+extern __thread bool mysql_thread___firewall_whitelist_enabled;
 extern __thread bool mysql_thread___use_tcp_keepalive;
 extern __thread int mysql_thread___tcp_keepalive_time;
 extern __thread int mysql_thread___throttle_connections_per_sec_to_hostgroup;
@@ -821,13 +871,15 @@ extern __thread int mysql_thread___set_query_lock_on_hostgroup;
 extern __thread int mysql_thread___reset_connection_algorithm;
 extern __thread uint32_t mysql_thread___server_capabilities;
 extern __thread int mysql_thread___auto_increment_delay_multiplex;
-extern __thread uint8_t mysql_thread___default_charset;
+extern __thread unsigned int mysql_thread___handle_unknown_charset;
 extern __thread int mysql_thread___poll_timeout;
 extern __thread int mysql_thread___poll_timeout_on_failure;
+extern __thread bool mysql_thread___connection_warming;
 extern __thread bool mysql_thread___have_compress;
 extern __thread bool mysql_thread___have_ssl;
 extern __thread bool mysql_thread___client_found_rows;
 extern __thread bool mysql_thread___multiplexing;
+extern __thread bool mysql_thread___log_unhealthy_connections;
 extern __thread bool mysql_thread___forward_autocommit;
 extern __thread bool mysql_thread___enforce_autocommit_on_reads;
 extern __thread bool mysql_thread___autocommit_false_not_reusable;
@@ -851,6 +903,7 @@ extern __thread bool mysql_thread___session_idle_show_processlist;
 extern __thread bool mysql_thread___sessions_sort;
 extern __thread bool mysql_thread___kill_backend_connection_when_disconnect;
 extern __thread bool mysql_thread___client_session_track_gtid;
+extern __thread char * mysql_thread___default_variables[SQL_NAME_LAST];
 
 /* variables used for Query Cache */
 extern __thread int mysql_thread___query_cache_size_MB;
@@ -889,6 +942,7 @@ extern __thread int mysql_thread___monitor_replication_lag_timeout;
 extern __thread int mysql_thread___monitor_groupreplication_healthcheck_interval;
 extern __thread int mysql_thread___monitor_groupreplication_healthcheck_timeout;
 extern __thread int mysql_thread___monitor_groupreplication_healthcheck_max_timeout_count;
+extern __thread int mysql_thread___monitor_groupreplication_max_transactions_behind_count;
 extern __thread int mysql_thread___monitor_galera_healthcheck_interval;
 extern __thread int mysql_thread___monitor_galera_healthcheck_timeout;
 extern __thread int mysql_thread___monitor_galera_healthcheck_max_timeout_count;
@@ -909,3 +963,57 @@ extern __thread bool mysql_thread___session_debug;
 #endif /* DEBUG */
 extern __thread unsigned int g_seed;
 #endif /* PROXYSQL_EXTERN */
+
+#ifndef MYSQL_TRACKED_VARIABLES
+#define MYSQL_TRACKED_VARIABLES
+#ifdef PROXYSQL_EXTERN
+// field_1: index number
+// field_2: what status should be changed after setting this variables
+// field_3: if the variable needs to be quoted
+// field_4: if related to SET TRANSACTION statement . if false , it will be execute "SET varname = varvalue" . If true, "SET varname varvalue"
+// field_5: if true, some special handling is required
+// field_6: if true, the variable is a number. Special cases should be checked
+// field_7: if true, the variable is a boolean. Special cases should be checked
+// field_8: what variable name (or string) will be used when setting it to backend
+// field_9: variable name as displayed in admin , WITHOUT "default_"
+// field_10: default value
+/*
+typedef struct {
+	enum variable_name idx;     // index number
+	enum session_status status; // what status should be changed after setting this variables
+	bool quote;                 // if the variable needs to be quoted
+	bool set_transaction;       // if related to SET TRANSACTION statement . if false , it will be execute "SET varname = varvalue" . If true, "SET varname varvalue"
+	bool special_handling;      // if true, some special handling is required
+	bool is_number;				// if true, the variable is a number. Special cases should be checked
+	bool is_bool;				// if true, the variable is a boolean. Special cases should be checked
+	char * set_variable_name;   // what variable name (or string) will be used when setting it to backend
+	char * internal_variable_name; // variable name as displayed in admin , WITHOUT "default_"
+	char * default_value;       // default value
+} mysql_variable_st;
+*/
+mysql_variable_st mysql_tracked_variables[] {
+    { SQL_CHARACTER_SET,         SETTING_CHARSET,    false, true, false,  false, false, (char *)"CHARSET", (char *)"CHARSET", (char *)"UTF8" } , // should be before SQL_CHARACTER_SET_RESULTS
+    { SQL_CHARACTER_ACTION,      NONE,		         false, false, false, false, false, (char *)"action", (char *)"action", (char *)"1" } ,
+    { SQL_SET_NAMES,             SETTING_SET_NAMES,  false, false, false, false, false, (char *)"names", (char *)"names", (char *)"DEFAULT" } ,
+	{ SQL_SAFE_UPDATES,          SETTING_VARIABLE,   true,  false, true,  false, true,  (char *)"sql_safe_updates",  (char *)"sql_safe_updates", (char *)"OFF" } ,
+    { SQL_SELECT_LIMIT,          SETTING_VARIABLE,   false, false, true,  true, false, (char *)"sql_select_limit", (char *)"sql_select_limit", (char *)"DEFAULT" } ,
+	{ SQL_SQL_MODE,              SETTING_VARIABLE,   true,  false, true,  false, false, (char *)"sql_mode" , (char *)"sql_mode" , (char *)"" } ,
+    { SQL_TIME_ZONE,             SETTING_VARIABLE,   true,  false, true,  false, false, (char *)"time_zone", (char *)"time_zone", (char *)"SYSTEM" } ,
+	{ SQL_CHARACTER_SET_RESULTS, SETTING_VARIABLE,   false, false, true,  false, false, (char *)"character_set_results", (char *)"character_set_results", (char *)"UTF8" } ,
+	{ SQL_CHARACTER_SET_CONNECTION, SETTING_VARIABLE, false, false, true, false, false, (char *)"character_set_connection", (char *)"character_set_connection", (char *)"UTF8" } ,
+	{ SQL_CHARACTER_SET_CLIENT,     SETTING_VARIABLE, false, false, true, false, false, (char *)"character_set_client", (char *)"character_set_client", (char *)"UTF8" } ,
+	{ SQL_CHARACTER_SET_DATABASE,   SETTING_VARIABLE, false, false, true, false, false, (char *)"character_set_database", (char *)"character_set_database", (char *)"UTF8" } ,
+	{ SQL_ISOLATION_LEVEL,  SETTING_ISOLATION_LEVEL,  false, true,  true, false, false, (char *)"SESSION TRANSACTION ISOLATION LEVEL", (char *)"isolation_level", (char *)"READ COMMITTED" } ,
+	{ SQL_TRANSACTION_READ, SETTING_TRANSACTION_READ, false, true,  true, false, false, (char *)"SESSION TRANSACTION READ", (char *)"transaction_read", (char *)"WRITE" } ,
+    { SQL_SQL_AUTO_IS_NULL,     SETTING_VARIABLE,     true,  false, true, false, true,  (char *)"sql_auto_is_null", (char *)"sql_auto_is_null", (char *)"OFF" } ,
+    { SQL_COLLATION_CONNECTION, SETTING_VARIABLE,     true,  false, true, false, false, (char *)"collation_connection", (char *)"collation_connection", (char *)"utf8_general_ci" } ,
+    { SQL_NET_WRITE_TIMEOUT,    SETTING_VARIABLE,     false, false, true, true,  false, (char *)"net_write_timeout", (char *)"net_write_timeout", (char *)"60" } ,
+    { SQL_MAX_JOIN_SIZE,        SETTING_VARIABLE,     false, false, true, true,  false, (char *)"max_join_size", (char *)"max_join_size", (char *)"18446744073709551615" } ,
+    { SQL_LOG_BIN,              SETTING_VARIABLE,     false, false, true, false, false, (char *)"sql_log_bin", (char *)"sql_log_bin", (char *)"1" } ,
+    { SQL_WSREP_SYNC_WAIT,      SETTING_VARIABLE,     false, false, true, true,  false, (char *)"wsrep_sync_wait", (char *)"wsrep_sync_wait", (char *)"0" } ,
+    { SQL_GROUP_CONCAT_MAX_LEN, SETTING_VARIABLE,     false, false, true, true,  false, (char *)"group_concat_max_len", (char *)"group_concat_max_len", (char *)"1024" } ,
+};
+#else
+extern mysql_variable_st mysql_tracked_variables[];
+#endif // PROXYSQL_EXTERN
+#endif // MYSQL_TRACKED_VARIABLES
