@@ -1,6 +1,12 @@
 #include <fstream>
 #include "proxysql.h"
 #include "cpp.h"
+
+#include "MySQL_Data_Stream.h"
+#include "query_processor.h"
+#include "MySQL_PreparedStatement.h"
+#include "MySQL_Logger.hpp"
+
 #include <dirent.h>
 #include <libgen.h>
 
@@ -228,8 +234,9 @@ void MySQL_Event::write_auth(std::fstream *f, MySQL_Session *sess) {
 	}
 	// for performance reason, we are moving the write lock
 	// right before the write to disk
-	GloMyLogger->wrlock();
-	*f << j.dump() << std::endl;
+	//GloMyLogger->wrlock();
+	//move wrlock() function to log_audit_entry() function, avoid to get a null pointer in a multithreaded environment
+	*f << j.dump(-1, ' ', false, json::error_handler_t::replace) << std::endl;
 }
 
 uint64_t MySQL_Event::write_query_format_1(std::fstream *f) {
@@ -259,7 +266,8 @@ uint64_t MySQL_Event::write_query_format_1(std::fstream *f) {
 
 	// for performance reason, we are moving the write lock
 	// right before the write to disk
-	GloMyLogger->wrlock();
+	//GloMyLogger->wrlock();
+        //move wrlock() function to log_request() function, avoid to get a null pointer in a multithreaded environment
 
 	// write total length , fixed size
 	f->write((const char *)&total_bytes,sizeof(uint64_t));
@@ -400,9 +408,10 @@ uint64_t MySQL_Event::write_query_format_2_json(std::fstream *f) {
 
 	// for performance reason, we are moving the write lock
 	// right before the write to disk
-	GloMyLogger->wrlock();
+	//GloMyLogger->wrlock();
+        //move wrlock() function to log_request() function, avoid to get a null pointer in a multithreaded environment
 
-	*f << j.dump() << std::endl;
+	*f << j.dump(-1, ' ', false, json::error_handler_t::replace) << std::endl;
 	return total_bytes; // always 0
 }
 
@@ -578,6 +587,8 @@ void MySQL_Logger::events_set_base_filename() {
 }
 
 void MySQL_Logger::events_set_datadir(char *s) {
+	if (events.datadir)
+		free(events.datadir);
 	events.datadir=strdup(s);
 	flush_log();
 };
@@ -606,6 +617,8 @@ void MySQL_Logger::audit_set_base_filename() {
 }
 
 void MySQL_Logger::audit_set_datadir(char *s) {
+	if (audit.datadir)
+		free(audit.datadir);
 	audit.datadir=strdup(s);
 	flush_log();
 };
@@ -707,6 +720,9 @@ void MySQL_Logger::log_request(MySQL_Session *sess, MySQL_Data_Stream *myds) {
 	// for performance reason, we are moving the write lock
 	// right before the write to disk
 	//wrlock();
+	
+	//add a mutex lock in a multithreaded environment, avoid to get a null pointer of events.logfile that leads to the program coredump
+        GloMyLogger->wrlock();
 
 	me.write(events.logfile, sess);
 
@@ -860,6 +876,8 @@ void MySQL_Logger::log_audit_entry(log_event_type _et, MySQL_Session *sess, MySQ
 	// right before the write to disk
 	//wrlock();
 
+	//add a mutex lock in a multithreaded environment, avoid to get a null pointer of events.logfile that leads to the program coredump
+        GloMyLogger->wrlock();
 	me.write(audit.logfile, sess);
 
 
