@@ -2,6 +2,14 @@
 #include "proxysql.h"
 #include "cpp.h"
 
+#include "MySQL_PreparedStatement.h"
+#include "MySQL_Data_Stream.h"
+#include "MySQL_Authentication.hpp"
+#include "MySQL_LDAP_Authentication.hpp"
+#include "MySQL_Variables.h"
+
+#include <sstream>
+
 extern MySQL_Authentication *GloMyAuth;
 extern MySQL_LDAP_Authentication *GloMyLdapAuth;
 extern MySQL_Threads_Handler *GloMTH;
@@ -26,6 +34,7 @@ typedef uint8_t uchar;
 #endif
 
 extern const MARIADB_CHARSET_INFO * proxysql_find_charset_nr(unsigned int nr);
+MARIADB_CHARSET_INFO * proxysql_find_charset_name(const char *name);
 
 #ifdef DEBUG
 static void __dump_pkt(const char *func, unsigned char *_ptr, unsigned int len) {
@@ -1147,7 +1156,7 @@ bool MySQL_Protocol::generate_pkt_initial_handshake(bool send, void **ptr, unsig
     //+ sizeof(glovars.server_language)
     //+ sizeof(glovars.server_status)
     + sizeof(mysql_thread___server_capabilities)/2
-    + sizeof(mysql_thread___default_charset)
+    + sizeof(uint8_t) // charset in handshake is 1 byte
     + sizeof(server_status)
     + 3 // unknown stuff
     + 10 // filler
@@ -1207,7 +1216,15 @@ bool MySQL_Protocol::generate_pkt_initial_handshake(bool send, void **ptr, unsig
 	mysql_thread___server_capabilities |= CLIENT_MYSQL | CLIENT_PLUGIN_AUTH | CLIENT_RESERVED;
 	(*myds)->myconn->options.server_capabilities=mysql_thread___server_capabilities;
   memcpy(_ptr+l,&mysql_thread___server_capabilities, sizeof(mysql_thread___server_capabilities)/2); l+=sizeof(mysql_thread___server_capabilities)/2;
-  memcpy(_ptr+l,&mysql_thread___default_charset, sizeof(mysql_thread___default_charset)); l+=sizeof(mysql_thread___default_charset);
+  const MARIADB_CHARSET_INFO *ci = NULL;
+  ci = proxysql_find_charset_name(mysql_thread___default_variables[SQL_CHARACTER_SET]);
+  if (!ci) {
+	  proxy_error("Cannot find character set for name [%s]. Configuration error. Check [%s] global variable.\n",
+			  mysql_thread___default_variables[SQL_CHARACTER_SET], mysql_tracked_variables[SQL_CHARACTER_SET].internal_variable_name);
+	  assert(0);
+  }
+  uint8_t uint8_charset = ci->nr & 255;
+  memcpy(_ptr+l,&uint8_charset, sizeof(uint8_charset)); l+=sizeof(uint8_charset);
   memcpy(_ptr+l,&server_status, sizeof(server_status)); l+=sizeof(server_status);
   memcpy(_ptr+l,"\x8f\x80\x15",3); l+=3;
   for (i=0;i<10; i++) { _ptr[l]=0x00; l++; } //filler
@@ -1475,7 +1492,7 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 	if (dump_pkt) { __dump_pkt(__func__,pkt,len); }
 #endif
 	bool ret=false;
-	uint8_t charset;
+	unsigned int charset;
 	uint32_t  capabilities = 0;
 	uint32_t  max_pkt;
 	uint32_t  pass_len;
@@ -1532,6 +1549,7 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 		//(*myds)->switching_auth_stage=2;
 		charset=(*myds)->tmp_charset;
 		proxy_debug(PROXY_DEBUG_MYSQL_PROTOCOL,2,"Session=%p , DS=%p . Encrypted: %d , switching_auth: %d, auth_plugin_id: %d\n", (*myds)->sess, (*myds), (*myds)->encrypted, (*myds)->switching_auth_stage, auth_plugin_id);
+		capabilities = (*myds)->myconn->options.client_flag;
 		goto __do_auth;
 	}
 
@@ -1539,6 +1557,7 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 	(*myds)->myconn->options.client_flag = capabilities;
 	pkt     += sizeof(uint32_t);
 	max_pkt  = CPY4(pkt);
+	(*myds)->myconn->options.max_allowed_pkt = max_pkt;
 	pkt     += sizeof(uint32_t);
 	charset  = *(uint8_t *)pkt;
 	if ( (*myds)->encrypted == false ) { // client wants to use SSL
@@ -1552,7 +1571,13 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 	}
 	// see bug #810
 	if (charset==0) {
-		charset=mysql_thread___default_charset;
+		const MARIADB_CHARSET_INFO *ci = NULL;
+		ci = proxysql_find_charset_name(mysql_thread___default_variables[SQL_CHARACTER_SET]);
+		if (!ci) {
+			proxy_error("Cannot find charset [%s]\n", mysql_thread___default_variables[SQL_CHARACTER_SET]);
+			assert(0);
+		}
+		charset=ci->nr;
 	}
 	(*myds)->tmp_charset=charset;
 	pkt     += 24;
@@ -1709,8 +1734,8 @@ __do_auth:
 		// reject connections from unknown charsets
 		const MARIADB_CHARSET_INFO * c = proxysql_find_charset_nr(charset);
 		if (!c) {
-			proxy_error("Client %s:%d is trying to use unknown charset %d. Disconnecting\n", (*myds)->addr.addr, (*myds)->addr.port, charset);
-			proxy_debug(PROXY_DEBUG_MYSQL_AUTH, 5, "Session=%p , DS=%p , user='%s' . Client %s:%d is trying to use unknown charset %d. Disconnecting\n", (*myds), (*myds)->sess, user, (*myds)->addr.addr, (*myds)->addr.port, charset);
+			proxy_error("Client %s:%d is trying to use unknown charset %u. Disconnecting\n", (*myds)->addr.addr, (*myds)->addr.port, charset);
+			proxy_debug(PROXY_DEBUG_MYSQL_AUTH, 5, "Session=%p , DS=%p , user='%s' . Client %s:%d is trying to use unknown charset %u. Disconnecting\n", (*myds), (*myds)->sess, user, (*myds)->addr.addr, (*myds)->addr.port, charset);
 			ret = false;
 			goto __exit_do_auth;
 		}
@@ -1746,10 +1771,10 @@ __do_auth:
 			((*myds)->sess->session_type == PROXYSQL_SESSION_ADMIN)
 		|| 
 			((*myds)->sess->session_type == PROXYSQL_SESSION_STATS)
-#if defined(TEST_AURORA) || defined(TEST_GALERA) || defined(TEST_GROUPREP)
+//#if defined(TEST_AURORA) || defined(TEST_GALERA) || defined(TEST_GROUPREP)
 		|| 
 			((*myds)->sess->session_type == PROXYSQL_SESSION_SQLITE)
-#endif // TEST_AURORA  || TEST_GALERA
+//#endif // TEST_AURORA  || TEST_GALERA
 		) {
 			if (strcmp((const char *)user,mysql_thread___monitor_username)==0) {
 				proxy_scramble(reply, (*myds)->myconn->scramble_buff, mysql_thread___monitor_password);
@@ -1946,7 +1971,19 @@ __exit_do_auth:
 	assert(sess->client_myds);
 	myconn=sess->client_myds->myconn;
 	assert(myconn);
-	myconn->set_charset(charset);
+	myconn->set_charset(charset, CONNECT_START);
+	{
+		std::stringstream ss;
+		ss << charset;
+
+		/* We are processing handshake from client. Client sends us a character set it will use in communication.
+		 * we store this character set in the client's variables to use later in multiplexing with different backends
+		 */
+		mysql_variables.client_set_value(sess, SQL_CHARACTER_SET_RESULTS, ss.str().c_str());
+		mysql_variables.client_set_value(sess, SQL_CHARACTER_SET_CLIENT, ss.str().c_str());
+		mysql_variables.client_set_value(sess, SQL_CHARACTER_SET_CONNECTION, ss.str().c_str());
+		mysql_variables.client_set_value(sess, SQL_COLLATION_CONNECTION, ss.str().c_str());
+	}
 	// enable compression
 	if (capabilities & CLIENT_COMPRESS) {
 		if (myconn->options.server_capabilities & CLIENT_COMPRESS) {
@@ -1965,7 +2002,6 @@ __exit_do_auth:
 
 	if (ret==true) {
 
-		(*myds)->myconn->options.max_allowed_pkt=max_pkt;
 		(*myds)->DSS=STATE_CLIENT_HANDSHAKE;
 
 		if (!userinfo->username) // if set already, ignore
@@ -2304,7 +2340,6 @@ void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my,
 	resultset_completed=false;
 	myprot=_myprot;
 	mysql=_my;
-	MySQL_Data_Stream * c_myds = *(myprot->myds);
 	if (buffer==NULL) {
 	//if (_stmt==NULL) { // we allocate this buffer only for not prepared statements
 	// removing the previous assumption. We allocate this buffer also for prepared statements
@@ -2335,6 +2370,7 @@ void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my,
 	if (myprot==NULL) {
 		return; // this is a mirror
 	}
+	MySQL_Data_Stream * c_myds = *(myprot->myds);
 	if (c_myds->com_field_list==false) {
 		myprot->generate_pkt_column_count(false,&pkt.ptr,&pkt.size,sid,num_fields,this);
 		sid++;
@@ -2381,41 +2417,108 @@ void MySQL_ResultSet::init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my,
 		unsigned long long total_size=0;
 		MYSQL_ROWS *r=_stmt->result.data;
 		if (r) {
-		total_size+=r->length;
-		if (r->length > 0xFFFFFF) {
-			total_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
-		}
-		total_size+=sizeof(mysql_hdr);
-		while(r->next) {
-			r=r->next;
 			total_size+=r->length;
 			if (r->length > 0xFFFFFF) {
 				total_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
 			}
 			total_size+=sizeof(mysql_hdr);
-		}
-		PtrSize_t pkt;
-		pkt.size=total_size;
-		pkt.ptr=malloc(pkt.size);
-		total_size=0;
-		r=_stmt->result.data;
-		add_row2(r,(unsigned char *)pkt.ptr);
-		total_size+=r->length;
-		if (r->length > 0xFFFFFF) {
-			total_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
-		}
-		total_size+=sizeof(mysql_hdr);
-		while(r->next) {
-			r=r->next;
-			add_row2(r,(unsigned char *)pkt.ptr+total_size);
-			total_size+=r->length;
-			if (r->length > 0xFFFFFF) {
-				total_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
+			while(r->next) {
+				r=r->next;
+				total_size+=r->length;
+				if (r->length > 0xFFFFFF) {
+					total_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
+				}
+				total_size+=sizeof(mysql_hdr);
+			}
+#define MAXBUFFSTMT 12*1024*1024  // hardcoded to LESS *very important* than 16MB
+			if (total_size < MAXBUFFSTMT) {
+				PtrSize_t pkt;
+				pkt.size=total_size;
+				pkt.ptr=malloc(pkt.size);
+				total_size=0;
+				r=_stmt->result.data;
+				add_row2(r,(unsigned char *)pkt.ptr);
+				total_size+=r->length;
+				if (r->length > 0xFFFFFF) {
+					total_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
+				}
+				total_size+=sizeof(mysql_hdr);
+				while(r->next) {
+					r=r->next;
+					add_row2(r,(unsigned char *)pkt.ptr+total_size);
+					total_size+=r->length;
+					if (r->length > 0xFFFFFF) {
+						total_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
+					}
+					total_size+=sizeof(mysql_hdr);
+				}
+				PSarrayOUT.add(pkt.ptr,pkt.size);
+				if (resultset_size/0xFFFFFFF != ((resultset_size+pkt.size)/0xFFFFFFF)) {
+					// generate a heartbeat every 256MB
+					unsigned long long curtime=monotonic_time();
+					c_myds->sess->thread->atomic_curtime=curtime;
+				}
+				resultset_size+=pkt.size;
+			} else { // this code fixes a bug: resultset larger than 4GB would cause a crash
+				unsigned long long tmp_pkt_size = 0;
+				r=_stmt->result.data;
+				MYSQL_ROWS * r2 = NULL;
+				while (r) {
+					if (r->length >= MAXBUFFSTMT) {
+						// we have a large row
+						// we will send just that
+						tmp_pkt_size = r->length;
+						if (r->length > 0xFFFFFF) {
+							tmp_pkt_size+=(r->length / 0xFFFFFF) * sizeof(mysql_hdr);
+						}
+						tmp_pkt_size += sizeof(mysql_hdr);
+						PtrSize_t pkt;
+						pkt.size=tmp_pkt_size;
+						pkt.ptr=malloc(pkt.size);
+						add_row2(r,(unsigned char *)pkt.ptr);
+						PSarrayOUT.add(pkt.ptr,pkt.size);
+						if (resultset_size/0xFFFFFFF != ((resultset_size+pkt.size)/0xFFFFFFF)) {
+							// generate a heartbeat every 256MB
+							unsigned long long curtime=monotonic_time();
+							c_myds->sess->thread->atomic_curtime=curtime;
+						}
+						resultset_size+=pkt.size;
+						r=r->next; // next row
+					} else { // we have small row
+						r2 = r;
+						tmp_pkt_size = 0;
+						unsigned int a = 0;
+						while (r && (tmp_pkt_size + r->length) < MAXBUFFSTMT) {
+							a++;
+							tmp_pkt_size += r->length;
+							tmp_pkt_size += sizeof(mysql_hdr);
+							//if (r->next) {
+								r = r->next;
+							//}
+						}
+						r = r2; // we reset it back to the beginning
+						if (tmp_pkt_size) { // this should always be true
+							unsigned long long tmp2 = 0;
+							PtrSize_t pkt;
+							pkt.size=tmp_pkt_size;
+							pkt.ptr=malloc(pkt.size);
+							while (tmp2 < tmp_pkt_size) {
+								add_row2(r,(unsigned char *)pkt.ptr+tmp2);
+								tmp2 += r->length;
+								tmp2 += sizeof(mysql_hdr);
+								r = r->next;
+							}
+							PSarrayOUT.add(pkt.ptr,pkt.size);
+							if (resultset_size/0xFFFFFFF != ((resultset_size+pkt.size)/0xFFFFFFF)) {
+								// generate a heartbeat every 256MB
+								unsigned long long curtime=monotonic_time();
+								c_myds->sess->thread->atomic_curtime=curtime;
+							}
+							resultset_size+=pkt.size;
+						}
+					}
+				}
 			}
-			total_size+=sizeof(mysql_hdr);
-		}
-		PSarrayOUT.add(pkt.ptr,pkt.size);
-		resultset_size+=pkt.size;
 		}
 		add_eof();
 	}
@@ -2479,7 +2582,9 @@ unsigned int MySQL_ResultSet::add_row2(MYSQL_ROWS *row, unsigned char *offset) {
 			pkt_sid++;
 			memcpy(offset, &myhdr, sizeof(mysql_hdr));
 			offset+=sizeof(mysql_hdr);
-			memcpy(offset, row->data+copied, myhdr.pkt_length);
+			char *o = (char *) row->data;
+			o += copied;
+			memcpy(offset, o, myhdr.pkt_length);
 			offset+=0xFFFFFF;
 			// we are writing a large packet (over 16MB), we assume we are always outside the buffer
 			copied+=0xFFFFFF;
@@ -2491,7 +2596,9 @@ unsigned int MySQL_ResultSet::add_row2(MYSQL_ROWS *row, unsigned char *offset) {
 		pkt_sid++;
 		memcpy(offset, &myhdr, sizeof(mysql_hdr));
 		offset+=sizeof(mysql_hdr);
-		memcpy(offset, row->data+copied, myhdr.pkt_length);
+		char *o = (char *) row->data;
+		o += copied;
+		memcpy(offset, o, myhdr.pkt_length);
 		// we are writing a large packet (over 16MB), we assume we are always outside the buffer
 	}
 	sid=pkt_sid;
