@@ -5,6 +5,10 @@
 #include "re2/regexp.h"
 #include "proxysql.h"
 #include "cpp.h"
+#include "ProxySQL_HTTP_Server.hpp" // HTTP server
+#include "ProxySQL_Statistics.hpp"
+#include "SQLite3_Server.h"
+#include "MySQL_Authentication.hpp"
 
 #include <search.h>
 #include <stdlib.h>
@@ -79,12 +83,15 @@ static char * check_latest_version() {
 	curl_global_init(CURL_GLOBAL_ALL);
 
 	curl_handle = curl_easy_init();
-	curl_easy_setopt(curl_handle, CURLOPT_URL, "http://www.proxysql.com/latest");
+	curl_easy_setopt(curl_handle, CURLOPT_URL, "https://www.proxysql.com/latest");
 	curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
 	curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *)&chunk);
+	curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
 
-	curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "proxysql-agent/1.4.4");
-	curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, 5);
+	string s = "proxysql-agent/";
+	s += PROXYSQL_VERSION;
+	curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, s.c_str());
+	curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, 10);
 	curl_easy_setopt(curl_handle, CURLOPT_CONNECTTIMEOUT, 10);
 
 	res = curl_easy_perform(curl_handle);
