@@ -4,6 +4,8 @@
 #include "proxysql.h"
 #include "cpp.h"
 
+#include "ProxySQL_Statistics.hpp"
+
 //#include "thread.h"
 //#include "wqueue.h"
 
@@ -17,6 +19,7 @@
 #endif /* DEBUG */
 #define PROXYSQL_STATISTICS_VERSION "1.4.1027" DEB
 
+using namespace std;
 extern ProxySQL_Admin *GloAdmin;
 extern MySQL_Threads_Handler *GloMTH;
 
@@ -55,6 +58,7 @@ ProxySQL_Statistics::ProxySQL_Statistics() {
 	statsdb_disk->execute("PRAGMA synchronous=0");
 
 	next_timer_MySQL_Threads_Handler = 0;
+	next_timer_mysql_query_digest_to_disk = 0;
 	next_timer_system_cpu = 0;
 #ifndef NOJEM
 	next_timer_system_memory = 0;
@@ -77,6 +81,8 @@ void ProxySQL_Statistics::init() {
 	tables_defs_statsdb_disk = new std::vector<table_def_t *>;
 	insert_into_tables_defs(tables_defs_statsdb_mem,"mysql_connections", STATSDB_SQLITE_TABLE_MYSQL_CONNECTIONS);
 	insert_into_tables_defs(tables_defs_statsdb_disk,"mysql_connections", STATSDB_SQLITE_TABLE_MYSQL_CONNECTIONS);
+	insert_into_tables_defs(tables_defs_statsdb_disk,"history_mysql_status_variables", STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES);
+	insert_into_tables_defs(tables_defs_statsdb_disk,"history_stats_mysql_connection_pool", STATSDB_SQLITE_TABLE_HISTORY_STATS_MYSQL_CONNECTION_POOL);
 	insert_into_tables_defs(tables_defs_statsdb_disk,"system_cpu", STATSDB_SQLITE_TABLE_SYSTEM_CPU);
 #ifndef NOJEM
 	insert_into_tables_defs(tables_defs_statsdb_disk,"system_memory", STATSDB_SQLITE_TABLE_SYSTEM_MEMORY);
@@ -100,10 +106,20 @@ void ProxySQL_Statistics::init() {
 	insert_into_tables_defs(tables_defs_statsdb_disk,"myhgm_connections_hour", STATSDB_SQLITE_TABLE_MYHGM_CONNECTIONS_HOUR);
 	insert_into_tables_defs(tables_defs_statsdb_disk,"myhgm_connections_day", STATSDB_SQLITE_TABLE_MYHGM_CONNECTIONS_DAY);
 
+	insert_into_tables_defs(tables_defs_statsdb_disk,"history_mysql_query_digest", STATSDB_SQLITE_TABLE_HISTORY_MYSQL_QUERY_DIGEST);
+
 	disk_upgrade_mysql_connections();
 
 	check_and_build_standard_tables(statsdb_mem, tables_defs_statsdb_disk);
 	check_and_build_standard_tables(statsdb_disk, tables_defs_statsdb_disk);
+
+//	statsdb_disk->execute("CREATE INDEX IF NOT EXISTS idx_history_mysql_query_digest_hostgroup ON history_mysql_query_digest (hostgroup)");
+//	statsdb_disk->execute("CREATE INDEX IF NOT EXISTS idx_history_mysql_query_digest_username ON history_mysql_query_digest (username)");
+//	statsdb_disk->execute("CREATE INDEX IF NOT EXISTS idx_history_mysql_query_digest_schemaname ON history_mysql_query_digest (schemaname)");
+//	statsdb_disk->execute("CREATE INDEX IF NOT EXISTS idx_history_mysql_query_digest_digest ON history_mysql_query_digest (digest)");
+	statsdb_disk->execute("CREATE INDEX IF NOT EXISTS idx_history_mysql_query_digest_first_seen ON history_mysql_query_digest (first_seen)");
+//	statsdb_disk->execute("CREATE INDEX IF NOT EXISTS idx_history_mysql_query_digest_last_seen ON history_mysql_query_digest (last_seen)");
+	statsdb_disk->execute("CREATE INDEX IF NOT EXISTS idx_history_mysql_query_digest_dump_time ON history_mysql_query_digest (dump_time)");
 }
 
 void ProxySQL_Statistics::disk_upgrade_mysql_connections() {
@@ -201,6 +217,22 @@ bool ProxySQL_Statistics::MySQL_Query_Cache_timetoget(unsigned long long curtime
 	return false;
 }
 
+bool ProxySQL_Statistics::mysql_query_digest_to_disk_timetoget(unsigned long long curtime) {
+	unsigned int i = (unsigned int)variables.stats_mysql_query_digest_to_disk;
+	if (i) {
+		if (
+			( curtime > next_timer_mysql_query_digest_to_disk )
+			||
+			( curtime + i*1000*1000 < next_timer_mysql_query_digest_to_disk )
+		) {
+			next_timer_mysql_query_digest_to_disk = curtime/1000/1000 + i;
+			next_timer_mysql_query_digest_to_disk = next_timer_mysql_query_digest_to_disk * 1000 * 1000;
+			return true;
+		}
+	}
+	return false;
+}
+
 bool ProxySQL_Statistics::system_cpu_timetoget(unsigned long long curtime) {
 	unsigned int i = (unsigned int)variables.stats_system_cpu;
 	if (i) {
@@ -466,20 +498,16 @@ void ProxySQL_Statistics::system_cpu_sets() {
 		char *query1=NULL;
 		query1=(char *)"INSERT INTO system_cpu VALUES (?1, ?2, ?3)";
 		rc=sqlite3_prepare_v2(mydb3, query1, -1, &statement1, 0);
-		if (rc!=SQLITE_OK) {
-			proxy_error("SQLITE CRITICAL error: %s . Shutting down.\n", sqlite3_errmsg(mydb3));
-			exit(EXIT_SUCCESS);
-		}
-
+		ASSERT_SQLITE_OK(rc, statsdb_disk);
 		time_t ts = time(NULL);
 
-		rc = sqlite3_bind_int64(statement1, 1, ts); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 2, buf.tms_utime); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 3, buf.tms_stime); assert(rc==SQLITE_OK);
+		rc = sqlite3_bind_int64(statement1, 1, ts); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 2, buf.tms_utime); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 3, buf.tms_stime); ASSERT_SQLITE_OK(rc, statsdb_disk);
 
-		assert(rc==SQLITE_OK);
+		ASSERT_SQLITE_OK(rc, statsdb_disk);
 		SAFE_SQLITE3_STEP2(statement1);
-		rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
+		rc=sqlite3_clear_bindings(statement1); ASSERT_SQLITE_OK(rc, statsdb_disk);
 		rc=sqlite3_reset(statement1);
 		sqlite3_finalize(statement1);
 		
@@ -549,17 +577,17 @@ void ProxySQL_Statistics::system_memory_sets() {
 		mallctl("stats.retained", &retained, &sz, NULL, 0);
 
 
-		rc = sqlite3_bind_int64(statement1, 1, ts); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 2, allocated); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 3, resident); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 4, active); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 5, mapped); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 6, metadata); assert(rc==SQLITE_OK);
-		rc = sqlite3_bind_int64(statement1, 7, retained); assert(rc==SQLITE_OK);
+		rc = sqlite3_bind_int64(statement1, 1, ts); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 2, allocated); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 3, resident); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 4, active); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 5, mapped); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 6, metadata); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc = sqlite3_bind_int64(statement1, 7, retained); ASSERT_SQLITE_OK(rc, statsdb_disk);
 
-		assert(rc==SQLITE_OK);
+		ASSERT_SQLITE_OK(rc, statsdb_disk);
 		SAFE_SQLITE3_STEP2(statement1);
-		rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
+		rc=sqlite3_clear_bindings(statement1); ASSERT_SQLITE_OK(rc, statsdb_disk);
 		rc=sqlite3_reset(statement1);
 		sqlite3_finalize(statement1);
 
@@ -604,7 +632,49 @@ void ProxySQL_Statistics::system_memory_sets() {
 }
 #endif
 
-void ProxySQL_Statistics::MyHGM_Handler_sets(SQLite3_result *resultset) {
+void ProxySQL_Statistics::MyHGM_Handler_sets(SQLite3_result *resultset1, SQLite3_result *resultset2) {
+	MyHGM_Handler_sets_v1(resultset1);
+	if (GloVars.web_interface_plugin && resultset2) {
+		MySQL_Threads_Handler_sets_v2(resultset1);
+		MyHGM_Handler_sets_connection_pool(resultset2);
+	}
+}
+
+void ProxySQL_Statistics::MyHGM_Handler_sets_connection_pool(SQLite3_result *resultset) {
+	int rc;
+	if (resultset == NULL)
+		return;
+	sqlite3 *mydb3=statsdb_disk->get_db();
+	sqlite3_stmt *statement=NULL;
+	string query;
+	if (resultset->rows_count == 0) {
+		return;
+	}
+	time_t ts = time(NULL);
+	query = "INSERT INTO history_stats_mysql_connection_pool VALUES (?1";
+	for (int i=0; i < resultset->columns; i++) {
+		query += ",?" + to_string(i+2);
+	}
+	query += ")";
+	rc=sqlite3_prepare_v2(mydb3, query.c_str(), -1, &statement, 0);
+	if (rc!=SQLITE_OK) {
+		proxy_error("SQLITE CRITICAL error: %s . Shutting down.\n", sqlite3_errmsg(mydb3));
+		exit(EXIT_SUCCESS);
+	}
+	for (std::vector<SQLite3_row *>::iterator it = resultset->rows.begin() ; it != resultset->rows.end(); ++it) {
+		SQLite3_row *r=*it;
+		rc=sqlite3_bind_int64(statement, 1, ts); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		for (int i=0 ; i < resultset->columns ; i++) {
+			rc=sqlite3_bind_text(statement, i+2, r->fields[i] , -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		}
+		SAFE_SQLITE3_STEP2(statement);
+		rc=sqlite3_clear_bindings(statement); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc=sqlite3_reset(statement); //ASSERT_SQLITE_OK(rc, statsdb_disk);
+	}
+	sqlite3_finalize(statement);
+}
+
+void ProxySQL_Statistics::MyHGM_Handler_sets_v1(SQLite3_result *resultset) {
 	int rc;
 	if (resultset == NULL)
 		return;
@@ -622,9 +692,9 @@ void ProxySQL_Statistics::MyHGM_Handler_sets(SQLite3_result *resultset) {
 		exit(EXIT_SUCCESS);
 	}
 	//rc=sqlite3_prepare_v2(mydb3, query2, -1, &statement2, 0);
-	//assert(rc==SQLITE_OK);
+	//ASSERT_SQLITE_OK(rc, statsdb_disk);
 	//rc=sqlite3_prepare_v2(mydb3, query3, -1, &statement3, 0);
-	//assert(rc==SQLITE_OK);
+	//ASSERT_SQLITE_OK(rc, statsdb_disk);
 
 	time_t ts = time(NULL);
 
@@ -659,12 +729,12 @@ void ProxySQL_Statistics::MyHGM_Handler_sets(SQLite3_result *resultset) {
 	}
 
 	for (int i=0; i<6; i++) {
-		rc=sqlite3_bind_int64(statement1, i+1, myhgm_connections_values[i]); assert(rc==SQLITE_OK);
+		rc=sqlite3_bind_int64(statement1, i+1, myhgm_connections_values[i]); ASSERT_SQLITE_OK(rc, statsdb_disk);
 	}
 
 	SAFE_SQLITE3_STEP2(statement1);
-	rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
-	rc=sqlite3_reset(statement1); //assert(rc==SQLITE_OK);
+	rc=sqlite3_clear_bindings(statement1); ASSERT_SQLITE_OK(rc, statsdb_disk);
+	rc=sqlite3_reset(statement1); //ASSERT_SQLITE_OK(rc, statsdb_disk);
 	sqlite3_finalize(statement1);
 
 	SQLite3_result *resultset2 = NULL;
@@ -672,9 +742,6 @@ void ProxySQL_Statistics::MyHGM_Handler_sets(SQLite3_result *resultset) {
 	int affected_rows;
 	char *error = NULL;
 	char *query = NULL;
-	query = (char *)"SELECT MAX(timestamp) FROM mysql_connections_hour";
-	statsdb_disk->execute_statement(query, &error , &cols , &affected_rows , &resultset2);
-
 	query = (char *)"SELECT MAX(timestamp) FROM myhgm_connections_hour";
 	statsdb_disk->execute_statement(query, &error , &cols , &affected_rows , &resultset2);
 	if (error) {
@@ -711,6 +778,50 @@ void ProxySQL_Statistics::MyHGM_Handler_sets(SQLite3_result *resultset) {
 }
 
 void ProxySQL_Statistics::MySQL_Threads_Handler_sets(SQLite3_result *resultset) {
+	MySQL_Threads_Handler_sets_v1(resultset);
+	if (GloVars.web_interface_plugin) {
+		MySQL_Threads_Handler_sets_v2(resultset);
+	}
+}
+
+void ProxySQL_Statistics::MySQL_Threads_Handler_sets_v2(SQLite3_result *resultset) {
+	int rc;
+	if (resultset == NULL)
+		return;
+	sqlite3 *mydb3=statsdb_disk->get_db();
+	sqlite3_stmt *statement=NULL;
+	string query;
+	if (resultset->rows_count == 0) {
+		return;
+	}
+	time_t ts = time(NULL);
+	query = "INSERT INTO history_mysql_status_variables VALUES ";
+	int idx = 0;
+	for (int i=0; i < resultset->rows_count; i++) {
+		query += "(?" + to_string(idx+1) + ",?" + to_string(idx+2) + ",?" + to_string(idx+3) + "),";
+		idx+=3;
+	}
+	query.pop_back();
+	rc=sqlite3_prepare_v2(mydb3, query.c_str(), -1, &statement, 0);
+	if (rc!=SQLITE_OK) {
+		proxy_error("SQLITE CRITICAL error: %s . Shutting down.\n", sqlite3_errmsg(mydb3));
+		exit(EXIT_SUCCESS);
+	}
+	idx=0;
+	for (std::vector<SQLite3_row *>::iterator it = resultset->rows.begin() ; it != resultset->rows.end(); ++it) {
+		SQLite3_row *r=*it;
+		rc=sqlite3_bind_int64(statement, idx+1, ts); ASSERT_SQLITE_OK(rc, statsdb_disk);
+		rc=sqlite3_bind_text(statement, idx+2, r->fields[0] , -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, statsdb_disk); // name
+		rc=sqlite3_bind_text(statement, idx+3, r->fields[1] , -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, statsdb_disk); // value
+		idx+=3;
+	}
+	SAFE_SQLITE3_STEP2(statement);
+	rc=sqlite3_clear_bindings(statement); ASSERT_SQLITE_OK(rc, statsdb_disk);
+	rc=sqlite3_reset(statement); //ASSERT_SQLITE_OK(rc, statsdb_disk);
+	sqlite3_finalize(statement);
+}
+
+void ProxySQL_Statistics::MySQL_Threads_Handler_sets_v1(SQLite3_result *resultset) {
 	int rc;
 	if (resultset == NULL)
 		return;
@@ -729,9 +840,9 @@ void ProxySQL_Statistics::MySQL_Threads_Handler_sets(SQLite3_result *resultset)
 		exit(EXIT_SUCCESS);
 	}
 	//rc=sqlite3_prepare_v2(mydb3, query2, -1, &statement2, 0);
-	//assert(rc==SQLITE_OK);
+	//ASSERT_SQLITE_OK(rc, statsdb_disk);
 	//rc=sqlite3_prepare_v2(mydb3, query3, -1, &statement3, 0);
-	//assert(rc==SQLITE_OK);
+	//ASSERT_SQLITE_OK(rc, statsdb_disk);
 
 	time_t ts = time(NULL);
 
@@ -795,12 +906,12 @@ void ProxySQL_Statistics::MySQL_Threads_Handler_sets(SQLite3_result *resultset)
 	}
 
 	for (int i=0; i<13; i++) {
-		rc=sqlite3_bind_int64(statement1, i+1, mysql_connections_values[i]); assert(rc==SQLITE_OK);
+		rc=sqlite3_bind_int64(statement1, i+1, mysql_connections_values[i]); ASSERT_SQLITE_OK(rc, statsdb_disk);
 	}
 
 	SAFE_SQLITE3_STEP2(statement1);
-	rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
-	rc=sqlite3_reset(statement1); //assert(rc==SQLITE_OK);
+	rc=sqlite3_clear_bindings(statement1); ASSERT_SQLITE_OK(rc, statsdb_disk);
+	rc=sqlite3_reset(statement1); //ASSERT_SQLITE_OK(rc, statsdb_disk);
 	sqlite3_finalize(statement1);
 
 	SQLite3_result *resultset2 = NULL;
@@ -900,12 +1011,12 @@ void ProxySQL_Statistics::MySQL_Query_Cache_sets(SQLite3_result *resultset) {
 	}
 
 	for (int i=0; i<9; i++) {
-		rc=sqlite3_bind_int64(statement1, i+1, qc_values[i]); assert(rc==SQLITE_OK);
+		rc=sqlite3_bind_int64(statement1, i+1, qc_values[i]); ASSERT_SQLITE_OK(rc, statsdb_disk);
 	}
 
 	SAFE_SQLITE3_STEP2(statement1);
-	rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
-	rc=sqlite3_reset(statement1); //assert(rc==SQLITE_OK);
+	rc=sqlite3_clear_bindings(statement1); ASSERT_SQLITE_OK(rc, statsdb_disk);
+	rc=sqlite3_reset(statement1); //ASSERT_SQLITE_OK(rc, statsdb_disk);
 	sqlite3_finalize(statement1);
 
 	SQLite3_result *resultset2 = NULL;
