@@ -2,8 +2,99 @@
 #include "cpp.h"
 #include "SpookyV2.h"
 #include <fcntl.h>
+#include <sstream>
+
+#include "MySQL_PreparedStatement.h"
+#include "MySQL_Data_Stream.h"
+#include "query_processor.h"
+#include "MySQL_Variables.h"
 
 extern const MARIADB_CHARSET_INFO * proxysql_find_charset_nr(unsigned int nr);
+MARIADB_CHARSET_INFO * proxysql_find_charset_name(const char *name);
+
+void Variable::fill_server_internal_session(json &j, int conn_num, int idx) {
+	if (idx == SQL_CHARACTER_SET_RESULTS || idx == SQL_CHARACTER_SET_CLIENT || idx == SQL_CHARACTER_SET_DATABASE) {
+		const MARIADB_CHARSET_INFO *ci = NULL;
+		if (!value)
+			ci = proxysql_find_charset_name(mysql_tracked_variables[idx].default_value);
+		else
+			ci = proxysql_find_charset_nr(atoi(value));
+		if (!ci) {
+			if (idx == SQL_CHARACTER_SET_RESULTS && (!strcasecmp("NULL", value) || !strcasecmp("binary", value))) {
+				j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
+			}
+			else {
+				proxy_error("Cannot find charset [%s] for variables %d\n", value, idx);
+				assert(0);
+			}
+		}
+
+		j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string((ci && ci->csname)?ci->csname:"");
+	} else if (idx == SQL_CHARACTER_SET_CONNECTION) {
+		const MARIADB_CHARSET_INFO *ci = NULL;
+		if (!value)
+			ci = proxysql_find_charset_name(mysql_tracked_variables[idx].default_value);
+		else
+			ci = proxysql_find_charset_nr(atoi(value));
+
+		j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string((ci && ci->csname)?ci->csname:"");
+	} else if (idx == SQL_COLLATION_CONNECTION) {
+		const MARIADB_CHARSET_INFO *ci = NULL;
+		if (!value)
+			ci = proxysql_find_charset_collate(mysql_tracked_variables[idx].default_value);
+		else
+			ci = proxysql_find_charset_nr(atoi(value));
+
+		j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string((ci && ci->name)?ci->name:"");
+	} else if (idx == SQL_LOG_BIN) {
+		if (!value)
+			j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = mysql_tracked_variables[idx].default_value;
+		else
+			j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string(!strcmp("1",value)?"ON":"OFF");
+	} else {
+		j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string(value?value:"");
+	}
+}
+
+void Variable::fill_client_internal_session(json &j, int idx) {
+	if (idx == SQL_CHARACTER_SET_RESULTS || idx == SQL_CHARACTER_SET_CLIENT || idx == SQL_CHARACTER_SET_DATABASE) {
+		const MARIADB_CHARSET_INFO *ci = NULL;
+		if (!value)
+			ci = proxysql_find_charset_name(mysql_tracked_variables[idx].default_value);
+		else
+			ci = proxysql_find_charset_nr(atoi(value));
+		if (!ci) {
+			if (idx == SQL_CHARACTER_SET_RESULTS && (!strcasecmp("NULL", value) || !strcasecmp("binary", value))) {
+				j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
+			}
+			else {
+				proxy_error("Cannot find charset [%s] for variables %d\n", value, idx);
+				assert(0);
+			}
+		}
+
+		j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
+
+	} else if (idx == SQL_CHARACTER_SET_CONNECTION) {
+		const MARIADB_CHARSET_INFO *ci = NULL;
+		ci = proxysql_find_charset_nr(atoi(value));
+		j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
+	} else if (idx == SQL_COLLATION_CONNECTION) {
+		const MARIADB_CHARSET_INFO *ci = NULL;
+		if (!value)
+			ci = proxysql_find_charset_collate(mysql_tracked_variables[idx].default_value);
+		else
+			ci = proxysql_find_charset_nr(atoi(value));
+		j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->name)?ci->name:"";
+	}  else if (idx == SQL_LOG_BIN) {
+		if (!value)
+			j["conn"][mysql_tracked_variables[idx].internal_variable_name] = mysql_tracked_variables[idx].default_value;
+		else
+			j["conn"][mysql_tracked_variables[idx].internal_variable_name] = !strcmp("1", value)?"ON":"OFF";
+	} else {
+		j["conn"][mysql_tracked_variables[idx].internal_variable_name] = value?value:"";
+	}
+}
 
 #define PROXYSQL_USE_RESULT
 
@@ -198,6 +289,12 @@ MySQL_Connection::MySQL_Connection() {
 	fd=-1;
 	status_flags=0;
 	last_time_used=0;
+
+	for (auto i = 0; i < SQL_NAME_LAST; i++) {
+		variables[i].value = NULL;
+		var_hash[i] = 0;
+	}
+
 	options.client_flag = 0;
 	options.compression_min_length=0;
 	options.server_version=NULL;
@@ -206,48 +303,12 @@ MySQL_Connection::MySQL_Connection() {
 	options.no_backslash_escapes=false;
 	options.init_connect=NULL;
 	options.init_connect_sent=false;
-	options.character_set_results = NULL;
-	options.isolation_level = NULL;
-	options.tx_isolation = NULL;
-	options.transaction_read = NULL;
 	options.session_track_gtids = NULL;
-	options.sql_auto_is_null = NULL;
-	options.sql_select_limit = NULL;
-	options.sql_safe_updates = NULL;
-	options.collation_connection = NULL;
-	options.net_write_timeout = NULL;
-	options.max_join_size = NULL;
-	options.isolation_level_sent = false;
-	options.tx_isolation_sent = false;
-	options.transaction_read_sent = false;
-	options.character_set_results_sent = false;
 	options.session_track_gtids_sent = false;
-	options.sql_auto_is_null_sent = false;
-	options.sql_select_limit_sent = false;
-	options.sql_safe_updates_sent = false;
-	options.collation_connection_sent = false;
-	options.net_write_timeout_sent = false;
-	options.max_join_size_sent = false;
-	options.sql_mode_sent=false;
 	options.ldap_user_variable=NULL;
 	options.ldap_user_variable_value=NULL;
 	options.ldap_user_variable_sent=false;
-	options.sql_log_bin=1;	// default #818
-	options.sql_mode=NULL;	// #509
-	options.sql_mode_int=0;	// #509
-	options.time_zone=NULL;	// #819
-	options.time_zone_int=0;	// #819
-	options.isolation_level_int=0;
-	options.tx_isolation_int=0;
-	options.transaction_read_int=0;
-	options.character_set_results_int=0;
 	options.session_track_gtids_int=0;
-	options.sql_auto_is_null_int=0;
-	options.sql_select_limit_int=0;
-	options.sql_safe_updates_int=0;
-	options.collation_connection_int=0;
-	options.net_write_timeout_int=0;
-	options.max_join_size_int=0;
 	compression_pkt_id=0;
 	mysql_result=NULL;
 	query.ptr=NULL;
@@ -316,46 +377,21 @@ MySQL_Connection::~MySQL_Connection() {
 	if (query.stmt) {
 		query.stmt=NULL;
 	}
-	if (options.sql_mode) {
-		free(options.sql_mode);
-		options.sql_mode=NULL;
-	}
-	if (options.time_zone) {
-		free(options.time_zone);
-		options.time_zone=NULL;
-	}
-	if (options.isolation_level) {
-		free(options.isolation_level);
-		options.isolation_level=NULL;
-	}
-	if (options.tx_isolation) {
-		free(options.tx_isolation);
-		options.tx_isolation=NULL;
-	}
-	if (options.transaction_read) {
-		free(options.transaction_read);
-		options.transaction_read=NULL;
-	}
-	if (options.character_set_results) {
-		free(options.character_set_results);
-		options.character_set_results=NULL;
-	}
+
 	if (options.session_track_gtids) {
 		free(options.session_track_gtids);
 		options.session_track_gtids=NULL;
 	}
-	if (options.sql_auto_is_null) {
-		free(options.sql_auto_is_null);
-		options.sql_auto_is_null=NULL;
-	}
-	if (options.sql_select_limit) {
-		free(options.sql_select_limit);
-		options.sql_select_limit=NULL;
-	}
-	if (options.sql_safe_updates) {
-		free(options.sql_safe_updates);
-		options.sql_safe_updates=NULL;
+
+	for (auto i = 0; i < SQL_NAME_LAST; i++) {
+		if (variables[i].value) {
+			free(variables[i].value);
+			variables[i].value = NULL;
+			var_hash[i] = 0;
+		}
 	}
+
+/*
 	if (options.collation_connection) {
 		free(options.collation_connection);
 		options.collation_connection=NULL;
@@ -368,6 +404,7 @@ MySQL_Connection::~MySQL_Connection() {
 		free(options.max_join_size);
 		options.max_join_size=NULL;
 	}
+*/
 };
 
 bool MySQL_Connection::set_autocommit(bool _ac) {
@@ -382,9 +419,23 @@ bool MySQL_Connection::set_no_backslash_escapes(bool _ac) {
 	return _ac;
 }
 
-uint8_t MySQL_Connection::set_charset(uint8_t _c) {
+void print_backtrace(void);
+
+unsigned int MySQL_Connection::set_charset(unsigned int _c, enum charset_action action) {
 	proxy_debug(PROXY_DEBUG_MYSQL_CONNPOOL, 4, "Setting charset %d\n", _c);
-	options.charset=_c;
+
+	// SQL_CHARACTER_SET should be set befor setting SQL_CHRACTER_ACTION
+	std::stringstream ss;
+	ss << _c;
+	mysql_variables.client_set_value(myds->sess, SQL_CHARACTER_SET, ss.str());
+
+	// When SQL_CHARACTER_ACTION is set character set variables are set according to
+	// SQL_CHRACTER_SET value
+	ss.str(std::string());
+	ss.clear();
+	ss << action;
+	mysql_variables.client_set_value(myds->sess, SQL_CHARACTER_ACTION, ss.str());
+
 	return _c;
 }
 
@@ -556,12 +607,36 @@ void MySQL_Connection::connect_start() {
 		mysql_ssl_set(mysql, mysql_thread___ssl_p2s_key, mysql_thread___ssl_p2s_cert, mysql_thread___ssl_p2s_ca, NULL, mysql_thread___ssl_p2s_cipher);
 	}
 	unsigned int timeout= 1;
+	const char *csname = NULL;
 	mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (void *)&timeout);
-	const MARIADB_CHARSET_INFO * c = proxysql_find_charset_nr(mysql_thread___default_charset);
+	/* Take client character set and use it to connect to backend */
+	if (myds && myds->sess) {
+		csname = mysql_variables.client_get_value(myds->sess, SQL_CHARACTER_SET);
+	}
+
+	const MARIADB_CHARSET_INFO * c = NULL;
+	if (csname)
+		c = proxysql_find_charset_nr(atoi(csname));
+	else
+		c = proxysql_find_charset_name(mysql_thread___default_variables[SQL_CHARACTER_SET]);
+
 	if (!c) {
-		proxy_error("Not existing charset number %u\n", mysql_thread___default_charset);
+		proxy_error("Not existing charset number %s\n", mysql_thread___default_variables[SQL_CHARACTER_SET]);
 		assert(0);
 	}
+	{
+		/* We are connecting to backend setting charset in mysql_options.
+		 * Client already has sent us a character set and client connection variables have been already set.
+		 * Now we store this charset in server connection variables to avoid updating this variables on backend.
+		 */
+		std::stringstream ss;
+		ss << c->nr;
+
+		mysql_variables.server_set_value(myds->sess, SQL_CHARACTER_SET_RESULTS, ss.str().c_str());
+		mysql_variables.server_set_value(myds->sess, SQL_CHARACTER_SET_CLIENT, ss.str().c_str());
+		mysql_variables.server_set_value(myds->sess, SQL_CHARACTER_SET_CONNECTION, ss.str().c_str());
+		mysql_variables.server_set_value(myds->sess, SQL_COLLATION_CONNECTION, ss.str().c_str());
+	}
 	mysql_options(mysql, MYSQL_SET_CHARSET_NAME, c->csname);
 	unsigned long client_flags = 0;
 	//if (mysql_thread___client_found_rows)
@@ -664,6 +739,18 @@ void MySQL_Connection::initdb_cont(short event) {
 	async_exit_status = mysql_select_db_cont(&interr,mysql, mysql_status(event, true));
 }
 
+void MySQL_Connection::set_option_start() {
+	PROXY_TRACE();
+
+	enum_mysql_set_option set_option;
+	set_option=((options.client_flag & CLIENT_MULTI_STATEMENTS) ? MYSQL_OPTION_MULTI_STATEMENTS_ON : MYSQL_OPTION_MULTI_STATEMENTS_OFF);
+	async_exit_status = mysql_set_server_option_start(&interr,mysql,set_option);
+}
+
+void MySQL_Connection::set_option_cont(short event) {
+	proxy_debug(PROXY_DEBUG_MYSQL_PROTOCOL, 6,"event=%d\n", event);
+	async_exit_status = mysql_set_server_option_cont(&interr,mysql, mysql_status(event, true));
+}
 
 void MySQL_Connection::set_autocommit_start() {
 	PROXY_TRACE();
@@ -677,12 +764,12 @@ void MySQL_Connection::set_autocommit_cont(short event) {
 
 void MySQL_Connection::set_names_start() {
 	PROXY_TRACE();
-	const MARIADB_CHARSET_INFO * c = proxysql_find_charset_nr(options.charset);
+	const MARIADB_CHARSET_INFO * c = proxysql_find_charset_nr(atoi(mysql_variables.client_get_value(myds->sess, SQL_CHARACTER_SET)));
 	if (!c) {
-		proxy_error("Not existing charset number %u\n", options.charset);
+		proxy_error("Not existing charset number %u\n", atoi(mysql_variables.client_get_value(myds->sess, SQL_CHARACTER_SET)));
 		assert(0);
 	}
-	async_exit_status = mysql_set_character_set_start(&interr,mysql, NULL, options.charset);
+	async_exit_status = mysql_set_character_set_start(&interr,mysql, NULL, atoi(mysql_variables.client_get_value(myds->sess, SQL_CHARACTER_SET)));
 }
 
 void MySQL_Connection::set_names_cont(short event) {
@@ -810,6 +897,14 @@ handler_again:
     break;
 			break;
 		case ASYNC_CONNECT_END:
+			if (myds) {
+				if (myds->sess) {
+					if (myds->sess->thread) {
+						unsigned long long curtime = monotonic_time();
+						myds->sess->thread->atomic_curtime=curtime;
+					}
+				}
+			}
 			if (!ret_mysql) {
 				// always increase the counter
 				proxy_error("Failed to mysql_real_connect() on %s:%d , FD (Conn:%d , MyDS:%d) , %d: %s.\n", parent->address, parent->port, mysql->net.fd , myds->fd, mysql_errno(mysql), mysql_error(mysql));
@@ -1260,7 +1355,8 @@ handler_again:
 			}
 			break;
 		case ASYNC_SET_AUTOCOMMIT_FAILED:
-			fprintf(stderr,"%s\n",mysql_error(mysql));
+			//fprintf(stderr,"%s\n",mysql_error(mysql));
+			proxy_error("Failed SET AUTOCOMMIT: %s\n",mysql_error(mysql));
 			break;
 		case ASYNC_SET_NAMES_START:
 			set_names_start();
@@ -1288,7 +1384,8 @@ handler_again:
 		case ASYNC_SET_NAMES_SUCCESSFUL:
 			break;
 		case ASYNC_SET_NAMES_FAILED:
-			fprintf(stderr,"%s\n",mysql_error(mysql));
+			//fprintf(stderr,"%s\n",mysql_error(mysql));
+			proxy_error("Failed SET NAMES: %s\n",mysql_error(mysql));
 			break;
 		case ASYNC_INITDB_START:
 			initdb_start();
@@ -1316,8 +1413,38 @@ handler_again:
 		case ASYNC_INITDB_SUCCESSFUL:
 			break;
 		case ASYNC_INITDB_FAILED:
-			fprintf(stderr,"%s\n",mysql_error(mysql));
+			proxy_error("Failed INITDB: %s\n",mysql_error(mysql));
+			//fprintf(stderr,"%s\n",mysql_error(mysql));
 			break;
+		case ASYNC_SET_OPTION_START:
+			set_option_start();
+			if (async_exit_status) {
+				next_event(ASYNC_SET_OPTION_CONT);
+			} else {
+				NEXT_IMMEDIATE(ASYNC_SET_OPTION_END);
+			}
+			break;
+		case ASYNC_SET_OPTION_CONT:
+			set_option_cont(event);
+			if (async_exit_status) {
+				next_event(ASYNC_SET_OPTION_CONT);
+			} else {
+				NEXT_IMMEDIATE(ASYNC_SET_OPTION_END);
+			}
+			break;
+		case ASYNC_SET_OPTION_END:
+			if (interr) {
+				NEXT_IMMEDIATE(ASYNC_SET_OPTION_FAILED);
+			} else {
+				NEXT_IMMEDIATE(ASYNC_SET_OPTION_SUCCESSFUL);
+			}
+			break;
+		case ASYNC_SET_OPTION_SUCCESSFUL:
+			break;
+		case ASYNC_SET_OPTION_FAILED:
+			proxy_error("Error setting MYSQL_OPTION_MULTI_STATEMENTS : %s\n", mysql_error(mysql));
+			break;
+
 		default:
 			assert(0); //we should never reach here
 			break;
@@ -1679,7 +1806,7 @@ int MySQL_Connection::async_set_autocommit(short event, bool ac) {
 	return 1;
 }
 
-int MySQL_Connection::async_set_names(short event, uint8_t c) {
+int MySQL_Connection::async_set_names(short event, unsigned int c) {
 	PROXY_TRACE();
 	assert(mysql);
 	assert(ret_mysql);
@@ -1693,7 +1820,8 @@ int MySQL_Connection::async_set_names(short event, uint8_t c) {
 			return -1;
 			break;
 		case ASYNC_IDLE:
-			set_charset(c);
+			/* useless statement. should be removed after thorough testing */
+			//set_charset(c, CONNECT_START);
 			async_state_machine=ASYNC_SET_NAMES_START;
 		default:
 			handler(event);
@@ -1717,6 +1845,47 @@ int MySQL_Connection::async_set_names(short event, uint8_t c) {
 	return 1;
 }
 
+int MySQL_Connection::async_set_option(short event, bool mask) {
+	PROXY_TRACE();
+	assert(mysql);
+	assert(ret_mysql);
+	switch (async_state_machine) {
+		case ASYNC_SET_OPTION_SUCCESSFUL:
+			unknown_transaction_status = false;
+			async_state_machine=ASYNC_IDLE;
+			return 0;
+			break;
+		case ASYNC_SET_OPTION_FAILED:
+			return -1;
+			break;
+		case ASYNC_IDLE:
+			if (mask)
+				options.client_flag |= CLIENT_MULTI_STATEMENTS;
+			else
+				options.client_flag &= ~CLIENT_MULTI_STATEMENTS;
+			async_state_machine=ASYNC_SET_OPTION_START;
+		default:
+			handler(event);
+			break;
+	}
+
+	// check again
+	switch (async_state_machine) {
+		case ASYNC_SET_OPTION_SUCCESSFUL:
+			unknown_transaction_status = false;
+			async_state_machine=ASYNC_IDLE;
+			return 0;
+			break;
+		case ASYNC_SET_OPTION_FAILED:
+			return -1;
+			break;
+		default:
+			return 1;
+			break;
+	}
+	return 1;
+}
+
 void MySQL_Connection::async_free_result() {
 	PROXY_TRACE();
 	assert(mysql);
@@ -1789,6 +1958,13 @@ bool MySQL_Connection::IsAutoCommit() {
 				// we assume that what we sent last is correct .  #873
 				ret = false;
 			}
+		} else {
+			if (options.last_set_autocommit==-1) {
+				// if a connection was reset (thus last_set_autocommit==-1)
+				// the information related to SERVER_STATUS_AUTOCOMMIT is lost
+				// therefore we fall back on the safe assumption that autocommit==1
+				ret = true;
+			}
 		}
 	}
 	return ret;
@@ -2079,89 +2255,20 @@ void MySQL_Connection::reset() {
 	status_flags=0;
 	reusable=true;
 	options.last_set_autocommit=-1; // never sent
-	{ // bug #1160
-		options.sql_mode_int = 0;
-		if (options.sql_mode) {
-			free(options.sql_mode);
-			options.sql_mode = NULL;
-			options.sql_mode_sent = false;
-			
-		}
-		options.time_zone_int = 0;
-		if (options.time_zone) {
-			free(options.time_zone);
-			options.time_zone = NULL;
-		}
-	}
+
 	delete local_stmts;
 	local_stmts=new MySQL_STMTs_local_v14(false);
 	creation_time = monotonic_time();
-	options.isolation_level_int = 0;
-	if (options.isolation_level) {
-		free (options.isolation_level);
-		options.isolation_level = NULL;
-		options.isolation_level_sent = false;
-	}
-	options.tx_isolation_int = 0;
-	if (options.tx_isolation) {
-		free (options.tx_isolation);
-		options.tx_isolation = NULL;
-		options.tx_isolation_sent = false;
-	}
-	options.transaction_read_int = 0;
-	if (options.transaction_read) {
-		free (options.transaction_read);
-		options.transaction_read = NULL;
-		options.transaction_read_sent = false;
-	}
-	options.character_set_results_int = 0;
-	if (options.character_set_results) {
-		free (options.character_set_results);
-		options.character_set_results = NULL;
-		options.character_set_results_sent = false;
-	}
-	options.session_track_gtids_int = 0;
-	if (options.session_track_gtids) {
-		free (options.session_track_gtids);
-		options.session_track_gtids = NULL;
-		options.session_track_gtids_sent = false;
-	}
-	options.sql_auto_is_null_int = 0;
-	if (options.sql_auto_is_null) {
-		free (options.sql_auto_is_null);
-		options.sql_auto_is_null = NULL;
-		options.sql_auto_is_null_sent = false;
-	}
-	options.sql_select_limit_int = 0;
-	if (options.sql_select_limit) {
-		free (options.sql_select_limit);
-		options.sql_select_limit = NULL;
-		options.sql_select_limit_sent = false;
-	}
-	options.sql_safe_updates_int = 0;
-	if (options.sql_safe_updates) {
-		free (options.sql_safe_updates);
-		options.sql_safe_updates = NULL;
-		options.sql_safe_updates_sent = false;
-	}
-	options.collation_connection_int = 0;
-	if (options.collation_connection) {
-		free (options.collation_connection);
-		options.collation_connection = NULL;
-		options.collation_connection_sent = false;
-	}
-	options.net_write_timeout_int = 0;
-	if (options.net_write_timeout) {
-		free (options.net_write_timeout);
-		options.net_write_timeout = NULL;
-		options.net_write_timeout_sent = false;
-	}
-	options.max_join_size_int = 0;
-	if (options.max_join_size) {
-		free (options.max_join_size);
-		options.max_join_size = NULL;
-		options.max_join_size_sent = false;
+
+	for (auto i = 0; i < SQL_NAME_LAST; i++) {
+		var_hash[i] = 0;
+		if (variables[i].value) {
+			free(variables[i].value);
+			variables[i].value = NULL;
+			var_hash[i] = 0;
+		}
 	}
+
 	if (options.init_connect) {
 		free(options.init_connect);
 		options.init_connect = NULL;
@@ -2176,6 +2283,12 @@ void MySQL_Connection::reset() {
 		options.ldap_user_variable = NULL;
 		options.ldap_user_variable_sent = false;
 	}
+	options.session_track_gtids_int = 0;
+	if (options.session_track_gtids) {
+		free (options.session_track_gtids);
+		options.session_track_gtids = NULL;
+		options.session_track_gtids_sent = false;
+	}
 }
 
 bool MySQL_Connection::get_gtid(char *buff, uint64_t *trx_id) {
@@ -2190,6 +2303,9 @@ bool MySQL_Connection::get_gtid(char *buff, uint64_t *trx_id) {
 				const char *data;
 				size_t length;
 				if (mysql_session_track_get_first(mysql, SESSION_TRACK_GTIDS, &data, &length) == 0) {
+					if (length >= (sizeof(gtid_uuid) - 1)) {
+						length = sizeof(gtid_uuid) - 1;
+					}
 					if (memcmp(gtid_uuid,data,length)) {
 						// copy to local buffer in MySQL_Connection
 						memcpy(gtid_uuid,data,length);
