@@ -5,6 +5,12 @@
 #define UNIX_PATH_MAX    108
 #endif 
 
+#include "MySQL_PreparedStatement.h"
+#include "MySQL_Data_Stream.h"
+
+/*
+
+in libssl 1.1.0 
 struct bio_st {
 	const BIO_METHOD *method;
 	long (*callback) (struct bio_st *, int, const char *, int, long, long);
@@ -23,6 +29,32 @@ struct bio_st {
 	CRYPTO_EX_DATA ex_data;
 	CRYPTO_RWLOCK *lock;
 };
+*/
+
+typedef int CRYPTO_REF_COUNT;
+
+// in libssl 1.1.1
+struct bio_st {
+    const BIO_METHOD *method;
+    /* bio, mode, argp, argi, argl, ret */
+    BIO_callback_fn callback;
+    BIO_callback_fn_ex callback_ex;
+    char *cb_arg;               /* first argument for the callback */
+    int init;
+    int shutdown;
+    int flags;                  /* extra storage */
+    int retry_reason;
+    int num;
+    void *ptr;
+    struct bio_st *next_bio;    /* used by filter BIOs */
+    struct bio_st *prev_bio;    /* used by filter BIOs */
+    CRYPTO_REF_COUNT references;
+    uint64_t num_read;
+    uint64_t num_write;
+    CRYPTO_EX_DATA ex_data;
+    CRYPTO_RWLOCK *lock;
+};
+
 
 #define RESULTSET_BUFLEN_DS_16K 16000
 #define RESULTSET_BUFLEN_DS_1M 1000*1024
@@ -105,6 +137,7 @@ static void __dump_pkt(const char *func, unsigned char *_ptr, unsigned int len)
 static enum sslstatus get_sslstatus(SSL* ssl, int n)
 {
 	int err = SSL_get_error(ssl, n);
+	ERR_clear_error();
 	switch (err) {
 	case SSL_ERROR_NONE:
 		return SSLSTATUS_OK;
@@ -437,6 +470,7 @@ int MySQL_Data_Stream::read_from_net() {
 			r = SSL_read (ssl, queue_w_ptr(queueIN), s);
 		}
 */
+		PROXY_TRACE();
 		if (s < MY_SSL_BUFFER) {
 			return 0;	// no enough space for reads
 		}
@@ -444,7 +478,8 @@ int MySQL_Data_Stream::read_from_net() {
 		//ssize_t n = read(fd, buf, sizeof(buf));
 		int n = recv(fd, buf, sizeof(buf), 0);
 		//proxy_info("SSL recv of %d bytes\n", n);
-		if (n > 0) {
+		proxy_debug(PROXY_DEBUG_NET, 7, "Session=%p: recv() read %d bytes. num_write: %u ,  num_read: %u\n", sess, n,  rbio_ssl->num_write , rbio_ssl->num_read);
+		if (n > 0 || rbio_ssl->num_write > rbio_ssl->num_read) {
 			//on_read_cb(buf, (size_t)n);
 
 			char buf2[MY_SSL_BUFFER];
@@ -452,8 +487,9 @@ int MySQL_Data_Stream::read_from_net() {
 			enum sslstatus status;
 			char *src = buf;
 			int len = n;
-			while (len) {
+			while (len > 0) {
 				n2 = BIO_write(rbio_ssl, src, len);
+				proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p: write %d bytes into BIO %p, len=%d\n", sess, n2, rbio_ssl, len);
 				//proxy_info("BIO_write with len = %d and %d bytes\n", len , n2);
 				if (n2 <= 0) {
 					shut_soft();
@@ -477,6 +513,7 @@ int MySQL_Data_Stream::read_from_net() {
 				}
 			}
 			n2 = SSL_read (ssl, queue_w_ptr(queueIN), s);
+			proxy_debug(PROXY_DEBUG_NET, 5, "Session=%p: read %d bytes from BIO %p into a buffer with %d bytes free\n", sess, n2, rbio_ssl, s);
 			r = n2;
 			//proxy_info("Read %d bytes from SSL\n", r);
 			if (n2 > 0) {
@@ -843,6 +880,7 @@ int MySQL_Data_Stream::buffer2array() {
 			proxy_debug(PROXY_DEBUG_PKT_ARRAY, 5, "Session=%p . Reading the header of a new compressed packet\n", sess);
  			memcpy(&queueIN.hdr,queue_r_ptr(queueIN), sizeof(mysql_hdr));
 			queue_r(queueIN,sizeof(mysql_hdr));
+			pkt_sid=queueIN.hdr.pkt_id;
 			queueIN.pkt.size=queueIN.hdr.pkt_length+sizeof(mysql_hdr)+3;
 			queueIN.pkt.ptr=l_alloc(queueIN.pkt.size);
 			memcpy(queueIN.pkt.ptr, &queueIN.hdr, sizeof(mysql_hdr)); // immediately copy the header into the packet
@@ -1398,3 +1436,10 @@ void MySQL_Data_Stream::destroy_MySQL_Connection_From_Pool(bool sq) {
 	mc->send_quit=sq;
 	MyHGM->destroy_MyConn_from_pool(mc);
 }
+
+bool MySQL_Data_Stream::data_in_rbio() {
+	if (rbio_ssl->num_write > rbio_ssl->num_read) {
+		return true;
+	}
+	return false;
+}
